///////////////
// IanniX
///////////////

////////////////////////////
// NxScore.cpp
////////////////////////////

#include "NxScore.h"

//////////////////////////
// Constructors
//////////////////////////

// Empty constructor
NxScore::NxScore(QString inName, int inID, int inLength, int inHeight)
{
	// Initializing Object Counters
	cptCurve = cptTraject = cptCursor = cptTrigger = 0 ;
	maxOfIndexes() ;
	Color backColor, scoreColor, gridColor ;
	scoreColor.ID = backColor.ID = gridColor.ID = 0 ;
	scoreColor.c	= QColor(10,10,10,255) ;
	backColor.c		= QColor(0,0,0,255) ;
	gridColor.c		= QColor(255,255,255,255) ;
	////////////////////// GENERAL
	envData.xmlVersion	= "0.640b" ;
  	envData.name 		= inName ;
	envData.path		= "./" ;
  	envData.ID 			= inID ;
  	envData.groupID 	= 1 ; 
  	envData.length 		= inLength ;
  	envData.height 		= inHeight ;
	envData.nbCurves	= 0 ;
  	envData.nbTrajects	= 0 ;
  	envData.nbTriggers	= 0 ;
  	envData.nbCursors	= 0 ;
  	envData.nbComm 		= 0 ; // Comments
  	////////////////////// MODES
  	envData.visualMode		= true ;
	envData.oscOutMode		= true ;
	envData.oscInMode		= false ;
	envData.showTrajects	= true ;
	envData.showCursors		= true ;
	envData.showCurves		= true ;
	envData.showTriggers	= true ;
	////////////////////// OBJECTS DEFAULT VALUES
	envData.defaultSpatialPeriod		= 20 ;	// in ms
	envData.OSCTriggersDefaultIntValue	= 1 ;
	envData.mappingDefaultMin			= 0.0 ;
	envData.mappingDefaultMax			= 10.0 ;	
	////////////////////// FPS
	envData.displayStep					= 33 ;	// in ms
	envData.schedulerStep				= 4 ;	// in ms
	////////////////////// DISPLAY
	envData.backColor		= backColor ;
	envData.scoreColor		= scoreColor ;
	envData.gridColor		= gridColor ;
	envData.zoom			= 1 ;
	envData.showGrid		= true ;
	envData.crossesDraw		= false ;
	envData.scoreDraw		= true ;
	envData.displayOn		= false ;
	envData.circlesDraw		= false ;
	envData.rectDraw		= true ;
	envData.snapGrid		= false ;
	envData.autoGridStep	= false ;
	envData.gridStep		= envData.length / 10.0 ;
	envData.showCursorsArea = false ;
	////////////////////// TIME
	envData.useCirclesAlgo			= false ;
	envData.moveCursors				= true ;
	envData.globalTempo				= 1.0 ;
	envData.initialTime				= 0.0 ;
  	envData.finalTime				= 0.0 ;
	////////////////////// NETWORK
	envData.broadcastMode				= false ;
	envData.sendIFmessages				= false ;
	envData.UDPhost						= "localhost";
  	envData.UDPout						= 1979 ; 
  	envData.UDPin						= 1980 ;
	envData.OSCCurvesDefaultMFormat		= "/iannix/curves";
	envData.OSCTriggersDefaultMFormat	= "/iannix/triggers";
	/// Values to send
	envData.scCID = envData.scCGID = envData.scID = envData.scGID = envData.scA = envData.scV = true ;
	envData.stCID = envData.stCGID = envData.stID = envData.stGID = envData.stV = true ;
	envData.scCRID = envData.stCRID = envData.scRID = envData.stRID = false ;	// Real IDs
	envData.scX = envData.scY = envData.scZ = false ;							// Positions Curves
	envData.stX = envData.stY = envData.stZ = false ;							// Positions Triggers
	envData.OSCConfigMessageIP = "255.255.255.255" ;
}

////////////////////////////////////////////////////
// New Partition from XML file
////////////////////////////////////////////////////
NxScore::NxScore( QString inXMLpath ) 
{
	// Initializing XML parser
  	NxParser* parser = new NxParser( inXMLpath );
  
	NxEnv tempEnv = parser->parseEnv() ;
   	
	////// SCORE NAME //////
	int numb = inXMLpath.lastIndexOf("/") ;
	inXMLpath.remove(0,numb+1) ;
	if(inXMLpath.contains(".xml")) inXMLpath.chop(4) ;
	tempEnv.name = inXMLpath ;
	
	// Parsing XML File
  	setEnvData( tempEnv ) ;
	curves 		= parser->parseCurves() ;
  	trajects 	= parser->parseTrajects() ;
  	cursors 	= parser->parseCursors() ;
  	triggers 	= parser->parseTriggers() ;
	
	// Initializing Object Counters
	cptCurve	= tempEnv.nbCurves ;
	cptTraject	= tempEnv.nbTrajects ;
	cptCursor	= tempEnv.nbTriggers ;
	cptTrigger	= tempEnv.nbCursors ;
	
	maxOfIndexes() ;
}

///////////////////////
// Destructor
///////////////////////
NxScore::~NxScore(){}

/////////////////////////////////////////
// ACCESS METHODS
/////////////////////////////////////////
void NxScore::setEnvData( NxEnv inEnv )
{
	envData = inEnv;
}
void NxScore::setTempo( float inTempo )
{
	envData.globalTempo = inTempo;
}
void NxScore::setDisplayStep( int inDisStep )
{
	envData.displayStep = inDisStep;
}
void NxScore::setSchedulerStep( int inSchStep )
{
	envData.schedulerStep = inSchStep;
}
NxCurve* NxScore::getCurve( int inIndex )
{
	return curves.value(inIndex);
}
NxTraject* NxScore::getTraject( int inIndex )
{
	return trajects.value(inIndex);
}
NxCursor* NxScore::getCursor( int inIndex )
{
	return cursors.value(inIndex);
}
NxTrigger* NxScore::getTrigger( int inIndex )
{
	return triggers.value(inIndex);
}
NxEnv NxScore::getEnvironment()
{
	return envData;
}
float NxScore::getTempo()
{
	return envData.globalTempo;
}
QList<int> NxScore::curvesIndexes()
{
	return curves.keys();
}
QList<int> NxScore::trajectsIndexes()
{
	return trajects.keys();
}
QList<int> NxScore::triggersIndexes()
{
	return triggers.keys();
}
QList<int> NxScore::cursorsIndexes()
{
	return cursors.keys();
}

// Checks maximum existing index (to avoid conflicts)
void NxScore::maxOfIndexes() // could be better ?
{
	QList<int> curvesInd 	= curvesIndexes() ;
	QList<int> trajectsInd 	= trajectsIndexes() ;
	QList<int> cursorsInd 	= cursorsIndexes() ;
	QList<int> triggersInd	= triggersIndexes();

	// Sorting
	qSort(curvesInd);
	qSort(trajectsInd);
	qSort(cursorsInd);
	qSort(triggersInd);

	// Indexes
	if (!curvesInd.isEmpty()) 
		cptCurve = curvesInd.last() + 1;
	
	if (!trajectsInd.isEmpty())
		cptTraject = trajectsInd.last() + 1;
	
	if (!cursorsInd.isEmpty())
		cptCursor = cursorsInd.last() + 1;
	
	if (!triggersInd.isEmpty())
		cptTrigger = triggersInd.last() + 1;
}

////////////////////////////////////////////////////
// NEW OBJECTS METHODS
////////////////////////////////////////////////////

//////////////////
// CURVE
//////////////////
void NxScore::addCurve( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor1, Color inColor2, QVector<NxPoint> inCurveDef, int nbCtrlPoints, float inRMin, float inRMax, float inScale, float inAngle, int inSpatialPeriod, QString inHost, int inPort  )
{
  	QString tex = "default";
  	QString shape = "default";
  	NxCurve *newCurve = new NxCurve( inID, incID, inGroupID, inLay, inName, true, inPos, inColor1, inCurveDef, nbCtrlPoints, inColor2, true, tex, shape, inRMin, inRMax, inScale, inAngle, inSpatialPeriod, inHost, inPort );
	curves.insert(inID, newCurve);
	envData.nbCurves++;
}

/////////////////////
// TRIGGER
/////////////////////
void NxScore::addTrigger( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor, QString inHost, int inPort, int inIntValue)
{	
  	NxTrigger* trig = new  NxTrigger( inID, incID, inGroupID, inLay, inName, true, inPos, inColor, inHost, inPort, inIntValue);
  	triggers.insert(inID, trig);
  	envData.nbTriggers++;
}

/////////////////////
// TRAJECT
/////////////////////
void NxScore::addTraject( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor, int inType )
{
  	QList<int> myCursors;
  	NxTraject* traj = new NxTraject( inID, incID, inGroupID, inLay, inName, true, inPos, inColor, inType, 0, myCursors );
  	trajects.insert(inID, traj);
	envData.nbTrajects++;
}

/////////////////////
// CURSOR
/////////////////////
void NxScore::addCursor( NxPoint inPos, int inID, int incID, NxTraject* inParent, int inGroupID, int inLay, QString inName, Color inColor, bool inForward, int inMode, float nWidth, float inInitOffset, float inFinalOffset )
{
  	State* inSpeed = NULL;
  	State* inWidth = NULL;
  	bool inPlayF = false ;
  	NxCursor* cursor = new NxCursor( inID, incID, inGroupID, inLay, inParent, inName, true, inPos, inColor, inPlayF, inForward, inMode, inSpeed, inWidth, 0, 0, nWidth, inInitOffset, inFinalOffset );
  	cursors.insert(inID, cursor);
	inParent->addChild( inID );
  	envData.nbCursors++;
}

/////////////////////////
// REMOVING
/////////////////////////
void NxScore::removeObject( int inID, int inType )
{
  	switch( inType )
	{
		case 0 :
			curves.remove(inID);
			--envData.nbCurves ;
		break;
		case 1 :
      		trajects.remove(inID);
			--envData.nbTrajects ;
		break;
		case 2 :	
			cursors.remove(inID);	
			--envData.nbCursors ;
		break;	
		case 3 :
			triggers.remove(inID);
			--envData.nbTriggers ;
		break;
	}
}

////////////////////////////////////////////////
// Environment XML Writer
////////////////////////////////////////////////
void NxScore::writeEnvironment() 
{
	out << QString("\t<NxEnv>\n") ;
	out << QString("\t\t<XMLVersion v=\"0.640b\"/>\n") ;
  	out << QString("\t\t<ID name=\"%1\" ID=\"%2\" path=\"%3\" visualMode=\"%4\" oscOutMode=\"%5\" oscInMode=\"%6\"/>\n")
						.arg(envData.name)
						.arg(envData.ID)
						.arg(envData.path)
						.arg(envData.visualMode)
						.arg(envData.oscOutMode)
						.arg(envData.oscInMode) ;
						
	out << QString("\t\t<Bounds length=\"%1\" height=\"%2\"/>\n")
						.arg(envData.length)
						.arg(envData.height) ;
						
	out << QString("\t\t<Stats nbCurves=\"%1\" nbTrajects=\"%2\" nbTriggers=\"%3\" nbCursors=\"%4\"/>\n")
	   					.arg(envData.nbCurves)
						.arg(envData.nbTrajects)
						.arg(envData.nbTriggers)
						.arg(envData.nbCursors) ;	
						
  	out << QString("\t\t<Time circlesAlgo=\"%1\" globalTempo=\"%2\" startTime=\"%3\" stopTime=\"%4\" displayStep=\"%5\" schedulerStep=\"%6\"/>\n")
						.arg( envData.useCirclesAlgo )
						.arg( envData.globalTempo )
						.arg( envData.initialTime )
						.arg( envData.finalTime )
						.arg( envData.displayStep )
						.arg( envData.schedulerStep ) ;
						
	out << QString("\t\t<Network host=\"%1\" inPort=\"%2\" outPort=\"%3\" sendIFmessages=\"%4\" broadcastMode=\"%5\" curvesOSC=\"%6\" curvesArgs=\"%7%8%9%10%11%12%13%14%15%16%17\" triggersOSC=\"%18\" triggersArgs=\"%19%20%21%22%23%24%25%26%27%28\"/>\n")
		   				.arg(envData.UDPhost)
						.arg(envData.UDPin)
						.arg(envData.UDPout)
						.arg(envData.sendIFmessages)
						.arg(envData.broadcastMode)
						.arg(envData.OSCCurvesDefaultMFormat)
						.arg(envData.scCRID)
						.arg(envData.scCGID)
						.arg(envData.scCID)
						.arg(envData.scRID)
						.arg(envData.scGID)
						.arg(envData.scID)
						.arg(envData.scA)
						.arg(envData.scV)
						.arg(envData.scX)
						.arg(envData.scY)
						.arg(envData.scZ)
						.arg(envData.OSCTriggersDefaultMFormat)
						.arg(envData.stCRID)
						.arg(envData.stCGID)
						.arg(envData.stCID)
						.arg(envData.stRID)
						.arg(envData.stGID)
						.arg(envData.stID)
						.arg(envData.stV)
						.arg(envData.stX)
						.arg(envData.stY)
						.arg(envData.stZ) ;

	out << QString("\t\t<Display zoom=\"%1\" scoreDraw=\"%2\" displayOn=\"%3\" crossesDraw=\"%4\"/>\n")
		   				.arg(envData.zoom)
						.arg(envData.scoreDraw)
						.arg(envData.displayOn)
						.arg(envData.crossesDraw) ;
						
	Color c1 = envData.backColor ;
	Color c2 = envData.scoreColor ;
	Color c3 = envData.gridColor ;
	out << QString("\t\t<BackgroundColor ID=\"%1\" r=\"%2\" g=\"%3\" b=\"%4\" a=\"%5\"/>")
						.arg( QString().setNum(c1.ID) )
						.arg( QString().setNum(c1.c.red()) )
						.arg( QString().setNum(c1.c.green()) )
						.arg( QString().setNum(c1.c.blue()) )
						.arg( QString().setNum(c1.c.alpha()) ) << endl;
	out << QString("\t\t<ScoreColor ID=\"%1\" r=\"%2\" g=\"%3\" b=\"%4\" a=\"%5\"/>")
						.arg( QString().setNum(c2.ID) )
						.arg( QString().setNum(c2.c.red()) )
						.arg( QString().setNum(c2.c.green()) )
						.arg( QString().setNum(c2.c.blue()) )
						.arg( QString().setNum(c2.c.alpha()) ) << endl ;	
	out << QString("\t\t<GridColor ID=\"%1\" r=\"%2\" g=\"%3\" b=\"%4\" a=\"%5\"/>")
						.arg( QString().setNum(c3.ID) )
						.arg( QString().setNum(c3.c.red()) )
						.arg( QString().setNum(c3.c.green()) )
						.arg( QString().setNum(c3.c.blue()) )
						.arg( QString().setNum(c3.c.alpha()) ) << endl ;
  	
	out << QString("\t\t<Render texturing=\"1\" transparency=\"1\" background=\"0\"/>\n");
	out << QString("\t\t<Grid showGrid=\"%1\" snapGrid=\"%2\" autoGridStep=\"%3\" gridStep=\"%4\"/>\n")
		   				.arg(envData.showGrid)
						.arg(envData.snapGrid)
						.arg(envData.autoGridStep)
						.arg( QString().setNum(envData.gridStep) ) ;		
		
	out << QString("\t\t<Trajects show=\"%1\"/>\n")
		   				.arg(envData.showTrajects) ;
		   								
	out << QString("\t\t<Cursors show=\"%1\" showCursorsArea=\"%2\"/>\n")
						.arg(envData.showCursors)
		   				.arg(envData.showCursorsArea) ;
		   				
	out << QString("\t\t<Curves show=\"%1\" circlesDraw=\"%2\" rectDraw=\"%3\" mappingDefaultMin=\"%4\" mappingDefaultMax=\"%5\" defaultScanPeriod=\"%6\"/>\n")
						.arg(envData.showCurves)
						.arg(envData.circlesDraw)
						.arg(envData.rectDraw)
		   				.arg(envData.mappingDefaultMin)
						.arg(envData.mappingDefaultMax)
						.arg(envData.defaultSpatialPeriod) ;	

	out << QString("\t\t<Triggers show=\"%1\" defaultIntValue=\"%2\"/>\n")
						.arg(envData.showTriggers)
		   				.arg(envData.OSCTriggersDefaultIntValue) ;
  	
  	for( int i=0;i<envData.nbComm;i++ )
    	out << QString("\t\t<comment str=\"%1\"/>\n").arg(envData.comments[i]);
  	
	out << "\t</NxEnv>\n";
}

//////////////////////////////////////
// Curves XML Writer
//////////////////////////////////////
void NxScore::writeCurves()
{
	QString trans;
  	QVector<NxPoint> points;
  	Color c1, c2;
  	
	QList<int> indexes = curves.keys();
	
	for( int i=0; i<indexes.count(); i++)
	{
		int idx = indexes.at(i);
	
		if ( curves[idx]->isTransparent() ) 
			trans = "true";
		else 
			trans = "false";
	
		out << QString("\t<NxCurve ID=\"%1\" customID=\"%2\" groupID=\"%3\" layer=\"%4\" name=\"%5\" active=\"%6\" nbPoints=\"%7\" transparency=\"%8\"")
						.arg( curves[idx]->getID() )
						.arg( curves[idx]->getCustomID() )
						.arg( curves[idx]->getGroupID() )
						.arg( curves[idx]->getLayer() )
					   	.arg( curves[idx]->getName() )
						.arg( curves[idx]->isActive() )
						.arg( curves[idx]->size() )
						.arg( trans ) << endl;
		
		out << QString("\t\ttexture_path=\"%1\" shape=\"%2\" drawType=\"%3\" angle=\"%4\" realMin=\"%5\" realMax=\"%6\" scale=\"%7\" spatialPeriod=\"%8\" host=\"%9\" port=\"%10\">") 
						.arg( curves[idx]->getTexturePath() )
					    .arg( curves[idx]->getShape() )
						.arg( curves[idx]->getDrawType() )
						.arg( QString().setNum(curves[idx]->getAngle() ) )
						.arg( QString().setNum(curves[idx]->getRealMin() ) )
						.arg( QString().setNum(curves[idx]->getRealMax() ) )
						.arg( QString().setNum(curves[idx]->getScale() ) )
						.arg( curves[idx]->getSpatialPeriod() )
						.arg( curves[idx]->getHost() )
						.arg( QString().setNum(curves[idx]->getPort() ) ) << endl;
		
		// Colors
		c1 = curves[idx]->getCircleColor();
		c2 = curves[idx]->getColor();
		
		out << QString("\t\t<color ID=\"%1\" r=\"%2\" g=\"%3\" b=\"%4\" a=\"%5\"/>")
						.arg( QString().setNum(c1.ID) )
						.arg( QString().setNum(c1.c.red()) )
						.arg( QString().setNum(c1.c.green()) )
						.arg( QString().setNum(c1.c.blue()) )
						.arg( QString().setNum(c1.c.alpha()) ) << endl;

		out << QString("\t\t<color ID=\"%1\" r=\"%2\" g=\"%3\" b=\"%4\" a=\"%5\"/>")
					   	.arg( QString().setNum(c1.ID) )
						.arg( QString().setNum(c2.c.red()) )
						.arg( QString().setNum(c2.c.green()) )
						.arg( QString().setNum(c2.c.blue()) )
						.arg( QString().setNum(c2.c.alpha()) ) << endl; 
		
		// Curve position
		NxPoint pos = curves[idx]->getPosition() ;
		out << QString("\t\t<Position ID=\"%1\" x=\"%2\" y=\"%3\" z=\"%4\"/>")
					   	.arg( 0 )
						.arg( QString().setNum(pos.x) )
						.arg( QString().setNum(pos.y) )
						.arg( QString().setNum(pos.z) ) << endl; 
		// Points
		points = curves[idx]->getCurve();
		
		for( int j=0;j<curves[idx]->size();j++ )
			out << QString("\t\t<NxPoint ID=\"%1\" x=\"%2\" y=\"%3\" z=\"%4\"/>")
				   		.arg( points[j].ID )
					    .arg( QString().setNum(points[j].x) )
						.arg( QString().setNum(points[j].y) )
					    .arg( QString().setNum(points[j].z) ) << endl;
		
		out << "\t</NxCurve>" << endl;
	}
}

//////////////////////////////////////////////
// Trajectories XML Writer
//////////////////////////////////////////////
void NxScore::writeTrajects()
{
	Circle 		c;
  	NxPoint 	p1, p2;
  	QList<int> 	IDs;
	QList<int>	indexes = trajects.keys();
	Color c1 ;
	
  	for( int i=0; i<indexes.count(); i++ )
	{
		int idx = indexes.at(i);
		
		out << QString("\t<NxTraject ID=\"%1\" customID=\"%2\" groupID=\"%3\" layer=\"%4\" name=\"%5\" active=\"%6\" nbCursors=\"%7\">")
						.arg( trajects[idx]->getID() )
						.arg( trajects[idx]->getCustomID() )
						.arg( trajects[idx]->getGroupID() )
						.arg( trajects[idx]->getLayer() )
						.arg( trajects[idx]->getName() )
						.arg( trajects[idx]->isActive() )
						.arg( trajects[idx]->getNbCursors() ) << endl;
		// Color
		c1 = trajects[idx]->getColor();
		out << QString("\t\t<color ID=\"%1\" r=\"%2\" g=\"%3\" b=\"%4\" a=\"%5\"/>")
						.arg( QString().setNum(c1.ID) )
						.arg( QString().setNum(c1.c.red()) )
						.arg( QString().setNum(c1.c.green()) )
						.arg( QString().setNum(c1.c.blue()) )
						.arg( QString().setNum(c1.c.alpha()) ) << endl;
		// Shape
		out << QString("\t\t<shape type=\"%1\">").arg( trajects[idx]->getType() ) << endl;
		
		// Trajectory Definition
		switch ( trajects[idx]->getType() )
		{
			case 1 :
	 			c = trajects[idx]->getCircle();
	  			out << QString("\t\t\t<circle ID=\"%1\" x=\"%2\" y=\"%3\" z=\"%4\" radius=\"%5\"/>")
								.arg( c.p.ID )
								.arg( QString().setNum(c.p.x) )
								.arg( QString().setNum(c.p.y) )
								.arg( QString().setNum(c.p.z) )
								.arg( QString().setNum(c.radius) ) << endl;
	  		break;
			case 2 :	
	  			p1 = trajects[idx]->getInitialPoint();
	  			p2 = trajects[idx]->getFinalPoint();
	  			
				out << QString("\t\t\t<Orientation type=\"%1\"/>")
								.arg( trajects[idx]->getOrientation() ) << endl;
				out << QString("\t\t\t<NxPoint ID=\"%1\" x=\"%2\" y=\"%3\" z=\"%4\"/>")
								.arg( p1.ID )
								.arg( QString().setNum(p1.x) )
								.arg( QString().setNum(p1.y) )
								.arg( QString().setNum(p1.z) ) << endl;
				out << QString("\t\t\t<NxPoint ID=\"%1\" x=\"%2\" y=\"%3\" z=\"%4\"/>") 
		   						.arg( p2.ID )
								.arg( QString().setNum(p2.x) )
							    .arg( QString().setNum(p2.y) )
								.arg( QString().setNum(p2.z) ) << endl;			
			break;
		}
		
		out << "\t\t</shape>" << endl;
								
		// Linked Cursors
		IDs = trajects[idx]->getCursorIDs();
		
		for(int j=0; j<IDs.count(); j++)
			writeCursor( IDs.at(j) );
		
		// Closing Tag
		out << "\t</NxTraject>" << endl;			
	}
}

////////////////////////////////////////
// Cursors XML Writer (TO UPDATE)
////////////////////////////////////////
void NxScore::writeCursor( int ind )
{
  	int j, nbStates;
  	State* states;
  	NxPoint p;
	Color c1 ;
	
  	//printf( "Verif cursor ID %i \n", cursors[ind]->getID() );	
  	out << QString("\t\t<NxCursor ID=\"%1\" customID=\"%2\" groupID=\"%3\" layer=\"%4\" parentID=\"%5\" name=\"%6\" active=\"%7\" playFinished=\"%8\" forward=\"%9\" mode=\"%10\" offset=\"%11\" init_offset=\"%12\" final_offset=\"%13\" width=\"%14\">")
					.arg( cursors[ind]->getID() )
					.arg( cursors[ind]->getCustomID() )
					.arg( cursors[ind]->getGroupID() )
					.arg( cursors[ind]->getLayer() )
					.arg( cursors[ind]->getParentTrajectory()->getID() )
					.arg( cursors[ind]->getName() )
					.arg( cursors[ind]->isActive() )
					.arg( cursors[ind]->getPlayFinished() )
					.arg( cursors[ind]->getForward() )
					.arg( QString().setNum(cursors[ind]->getMode() ) )
					.arg( QString().setNum(cursors[ind]->getOffset()) )
					.arg( QString().setNum(cursors[ind]->getInitOffset()) )
					.arg( QString().setNum(cursors[ind]->getFinalOffset()) )
					.arg( QString().setNum(cursors[ind]->getWidth()) ) << endl;						
	// Color
	c1 = cursors[ind]->getColor();
	out << QString("\t\t\t<color ID=\"%1\" r=\"%2\" g=\"%3\" b=\"%4\" a=\"%5\"/>")
						.arg( QString().setNum(c1.ID) )
						.arg( QString().setNum(c1.c.red()) )
						.arg( QString().setNum(c1.c.green()) )
						.arg( QString().setNum(c1.c.blue()) )
						.arg( QString().setNum(c1.c.alpha()) ) << endl;
	
  	// Start NxPoint
  	p = cursors[ind]->getPosition();
  	out << QString("\t\t\t<NxPoint ID=\"%1\" x=\"%2\" y=\"%3\" z=\"%4\"/>")
					.arg( p.ID )
					.arg( QString().setNum(p.x) )
					.arg( QString().setNum(p.y) )
					.arg( QString().setNum(p.z) ) << endl;		
	
	// Speed Factor
	out << QString("\t\t\t<factor coeff=\"%1\"/>")
					.arg( QString().setNum(cursors[ind]->getSpeed()) ) << endl;
				
  	// Speed Def
  	/*nbStates 	= cursors[ind]->getNbSpeed();
  	states 		= cursors[ind]->getSpeedCurve();
  	
	out << QString("\t\t\t<speed count=\"%1\">").arg(nbStates) << endl; 
  	
	for ( j=0;j<nbStates;j++)
    	out << QString(" \t\t\t\t<state type=\"%1\" duration=\"%2\" scale=\"%3\" coeff=\"%4\" frequency=\"%5\"/>")
	    				.arg( states[j].type )
						.arg( states[j].duration )
						.arg( QString().setNum(states[j].scale) )
						.arg( QString().setNum(states[j].coeff) )
						.arg( states[j].frequency ) << endl;			 
  	
	out << "\t\t\t</speed>" << endl;
	
  	// Width Def			
  	nbStates 	= cursors[ind]->getNbWidth();
  	states 		= cursors[ind]->getWidthCurve();
  	
	out << QString("\t\t\t<width count=\"%1\">").arg(nbStates) << endl; 
  	
	for ( j=0;j<nbStates;j++)
    	out << QString(" \t\t\t\t<state type=\"%1\" duration=\"%2\" scale=\"%3\" coeff=\"%4\" frequency=\"%5\"/>")
	    					.arg( states[j].type )
						    .arg( states[j].duration )
							.arg( QString().setNum(states[j].scale) )
						    .arg( QString().setNum(states[j].coeff) )
						    .arg( states[j].frequency ) << endl;
	
  	out << "\t\t\t</width>" << endl;*/
						
  	// Closing tag
  	out << "\t\t</NxCursor>" << endl;
	
}

////////////////////////////////////////
// Triggers XML Writer
////////////////////////////////////////
void NxScore::writeTriggers()
{
	NxPoint p;
	QList<int> indexes = triggers.keys();
	Color c1 ;

  	for(int i=0; i<indexes.count(); i++)
	{
		int idx = indexes.at(i);
		
		out << QString("\t<NxTrigger ID=\"%1\" customID=\"%2\" groupID=\"%3\" layer=\"%4\" name=\"%5\" active=\"%6\"")
						.arg( triggers[idx]->getID() )
						.arg( triggers[idx]->getCustomID() )
						.arg( triggers[idx]->getGroupID() )
						.arg( triggers[idx]->getLayer() )
						.arg( triggers[idx]->getName() )
						.arg( triggers[idx]->isActive() ) << endl;		
		
		out << QString("\t\thost=\"%1\" port=\"%2\" value=\"%3\">")
						.arg( triggers[idx]->getHost() )
						.arg( triggers[idx]->getPort() )
						.arg( triggers[idx]->getValue() ) << endl;
		// Color
		c1 = triggers[idx]->getColor();
		out << QString("\t\t<color ID=\"%1\" r=\"%2\" g=\"%3\" b=\"%4\" a=\"%5\"/>")
						.arg( QString().setNum(c1.ID) )
						.arg( QString().setNum(c1.c.red()) )
						.arg( QString().setNum(c1.c.green()) )
						.arg( QString().setNum(c1.c.blue()) )
						.arg( QString().setNum(c1.c.alpha()) ) << endl;
						
		
		p = triggers[idx]->getPosition();
		
		out << QString("\t\t<NxPoint ID=\"%1\" x=\"%2\" y=\"%3\" z=\"%4\"/>")
						.arg( p.ID )
						.arg( QString().setNum(p.x) )
						.arg( QString().setNum(p.y) )
						.arg( QString().setNum(p.z) ) << endl;

		out << "\t</NxTrigger>" << endl;
    }	
}

////////////////////////
// XML Writer
///////////////////////
void NxScore::saveToXML( QString inFilename )
{
	if (!inFilename.contains(".xml"))
		inFilename.append(".xml");
	
	// Opening output file for writing
  	outFile.setFileName(inFilename);
	outFile.open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text);
	out.setDevice(&outFile);
	
  	// Writing Initial Tag
	out << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" ;
  	out << "<NxScore>" << endl;
	
	QString nam = inFilename ;
	int iNb = inFilename.lastIndexOf("/") ;
	inFilename.truncate(iNb+1) ;
	envData.path = inFilename ;
	envData.name = (nam.remove(0,iNb+1)).remove(".xml") ;
  	// Writing objects
  	writeEnvironment();
  	writeCurves();
  	writeTrajects();
  	writeTriggers();
	
  	// Last tag & file closing
  	out << "</NxScore>" << endl;
	outFile.close();
}
/////////////////////////////////////////////
/// Get the last ID for object creation
/////////////////////////////////////////////
int NxScore::getCptTraject()
{
	return cptTraject ;
}
int NxScore::getCptCurve()
{
	return cptCurve ;
}
int NxScore::getCptCursor()
{
	return cptCursor ;
}
int NxScore::getCptTrigger()
{
	return cptTrigger ;
}
/////////////////////////////////////////////
/// To know if an object exists ...
/////////////////////////////////////////////
bool NxScore::isThereTrigger( int inIndex )
{
	return triggers.contains( inIndex ) ;
}
bool NxScore::isThereTraject( int inIndex ) 
{
	return trajects.contains( inIndex ) ;
}
bool NxScore::isThereCursor( int inIndex ) 
{
	return cursors.contains( inIndex ) ;
}
bool NxScore::isThereCurve( int inIndex )
{
	return curves.contains( inIndex ) ;
}
