///////////////
// IanniX
///////////////

////////////////////////////
// NxListener
////////////////////////////

#include "NxListener.h"

NxListener::NxListener( NxGLScore *parent )
{ 
	pScore = (NxGLScore*)parent ;
}

void NxListener::ProcessBundle( const osc::ReceivedBundle& b, const IpEndpointName& remoteEndpoint )
{
	//printf("OSC Incoming Bundle Message processing...\n") ;
    for( osc::ReceivedBundle::const_iterator i = b.ElementsBegin(); i != b.ElementsEnd(); ++i ) {
        if( i->IsBundle() ) {
			//printf("OSC - Bundle inclu\n") ;
            osc::ReceivedBundle b(*i);
            this->ProcessBundle( b, 0 ) ;
        }
		else {
			//printf("OSC - Process Bundle message\n") ;
            osc::ReceivedMessage m(*i) ;
			this->ProcessMessage( m, 0 ) ;
        }
    }
}

void NxListener::ProcessMessage( const osc::ReceivedMessage& m, const IpEndpointName& )
{
	try {
		//printf("OSC Incoming Message processing...\n") ;
		////////////////////////////////////////////////////////////////////
		///////////////// TRIGGERS
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/new" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			float x = (arg++)->AsFloat();
			float y = (arg++)->AsFloat();
			float z = (arg++)->AsFloat();
			int cid = (arg++)->AsInt32();
			int gid = (arg++)->AsInt32();
			int val = (arg++)->AsInt32();
			int colR = (arg++)->AsInt32();
			int colG = (arg++)->AsInt32();
			int colB = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscNewTrigger( x, y, z, cid, gid, val, colR, colG, colB ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/value" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int val = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscTriggerValue( id, val ) ;
		}	
		////////////////////////////////////////////////////////////////////
		/////////////////// CURVE CREATION /////////////////////////////////
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/add/begin" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			float ix = (arg++)->AsFloat();
			float iy = (arg++)->AsFloat();
			float iz = (arg++)->AsFloat();
			int cid = (arg++)->AsInt32();
			int gid = (arg++)->AsInt32();
			int col1R = (arg++)->AsInt32();
			int col1G = (arg++)->AsInt32();
			int col1B = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscCurveCreationBegin( ix, iy, iz, cid, gid, col1R, col1G, col1B ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/add/pt" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			float px = (arg++)->AsFloat();
			float py = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscCurveCreationAddPt( px, py ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/add/end" ) == 0 ) {
			pScore->oscCurveCreationEnd() ;
		}
		////////////////////////////////////////////////////////////////////
		/////////////////// CURVE POINTS ///////////////////////////////
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/pt/new" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float ix = (arg++)->AsFloat();
			float iy = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscCurveAddPt( id, ix, iy ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/pt/del" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int num = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscCurveRmPt( id, num ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/pt/chg" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int nb = (arg++)->AsInt32();
			float ix = (arg++)->AsFloat();
			float iy = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscCurveModifPt( id, nb, ix, iy ) ;
		}
		////////////////////////////////////////////////////////////////////
		/////////////////// CURVE PARAMETERS ///////////////////////////////
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/min" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float min = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscCurveMin( id, min ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/max" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float max = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscCurveMax( id, max ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/angle" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float ang = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscCurveAngle( id, ang ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/scale" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float scale = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscCurveScale( id, scale ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/style" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int styl = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscCurveStyle( id, styl ) ;
		}
		////////////////////////////////////////////////////////////////////
		///////////////// TRAJECTS PARAMETERS
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/newcircle" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			float inx = (arg++)->AsFloat();
			float iny = (arg++)->AsFloat();
			float inz = (arg++)->AsFloat();
			float rad = (arg++)->AsFloat();
			int cid = (arg++)->AsInt32();
			int gid = (arg++)->AsInt32();
			int colR = (arg++)->AsInt32();
			int colG = (arg++)->AsInt32();
			int colB = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscNewCircle( inx, iny, inz, rad, cid, gid, colR, colG, colB ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/newline" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			float inx1 = (arg++)->AsFloat();
			float iny1 = (arg++)->AsFloat();
			float inz1 = (arg++)->AsFloat();
			float inx2 = (arg++)->AsFloat();
			float iny2 = (arg++)->AsFloat();
			float inz2 = (arg++)->AsFloat();
			int cid = (arg++)->AsInt32();
			int gid = (arg++)->AsInt32();
			int colR = (arg++)->AsInt32();
			int colG = (arg++)->AsInt32();
			int colB = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscNewLine( inx1, iny1, inz1, inx2, iny2, inz2, cid, gid, colR, colG, colB ) ;
		}		
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/radius" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float rad = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscTrajectRadius( id, rad ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/length" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float len = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscTrajectLength( id, len ) ;
		}		
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/pt" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int first = (arg++)->AsInt32();
			float inx = (arg++)->AsFloat();
			float iny = (arg++)->AsFloat();
			float inz = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscTrajectPoint( id, first, inx, iny, inz ) ;
		}
		////////////////////////////////////////////////////////////////////
		///////////////// CURSORS PARAMETERS
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/new" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int cid = (arg++)->AsInt32();
			int gid = (arg++)->AsInt32();
			float speed = (arg++)->AsFloat();
			float width = (arg++)->AsFloat();
			int forw = (arg++)->AsInt32();
			int mode = (arg++)->AsInt32();
			int colR = (arg++)->AsInt32();
			int colG = (arg++)->AsInt32();
			int colB = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscNewCursor( id, cid, gid, speed, width, forw, mode, colR, colG, colB ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/offset" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float off = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscCursorOffset( id, off ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/speed" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float speed = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscCursorSpeed( id, speed ) ;
		}	
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/width" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float wid = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscCursorWidth( id, wid ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/forward" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int playb = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscCursorForward( id, playb ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/mode" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int mode = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscCursorMode( id, mode ) ;
		}
		////////////////////////////////////////////////////////////////////
		///////////////// OBJECT DESTRUCTION
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/delete" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscDeleteObject( Cursor, id ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/delete" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscDeleteObject( Trigger, id ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/delete" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscDeleteObject( Curve, id ) ;
		}
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/delete" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
			
			pScore->oscDeleteObject( Trajectory, id ) ;
		}
		////////////////////////////////////////////////////////////////////
		///////////////// OBJECT POSITION
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/pos" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float newX = (arg++)->AsFloat();
			float newY = (arg++)->AsFloat();
			float newZ = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectPosition( Trajectory, id, newX, newY, newZ ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/pos" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float newX = (arg++)->AsFloat();
			float newY = (arg++)->AsFloat();
			float newZ = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectPosition( Curve, id, newX, newY, newZ ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/pos" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			float newX = (arg++)->AsFloat();
			float newY = (arg++)->AsFloat();
			float newZ = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectPosition( Trigger, id, newX, newY, newZ ) ;
		}
		////////////////////////////////////////////////////////////////////
		///////////////// OBJECT COLOR
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/color" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int newR = (arg++)->AsInt32();
			int newG = (arg++)->AsInt32();
			int newB = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectColor( Trajectory, id, newR, newG, newB ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/color" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int newR = (arg++)->AsInt32();
			int newG = (arg++)->AsInt32();
			int newB = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectColor( Cursor, id, newR, newG, newB ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/color" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int newR = (arg++)->AsInt32();
			int newG = (arg++)->AsInt32();
			int newB = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectColor( Curve, id, newR, newG, newB ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/color" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int newR = (arg++)->AsInt32();
			int newG = (arg++)->AsInt32();
			int newB = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectColor( Trigger, id, newR, newG, newB ) ;
		}
		////////////////////////////////////////////////////////////////////
		///////////////// OBJECT LAYER
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/layer" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int lay = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectLayer( Trajectory, id, lay ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/layer" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int lay = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectLayer( Cursor, id, lay ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/layer" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int lay = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectLayer( Curve, id, lay ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/layer" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int lay = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectLayer( Trigger, id, lay ) ;
		}
		////////////////////////////////////////////////////////////////////
		///////////////// OBJECT NAME
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/name" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			const char *nam = (arg++)->AsString();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectName( Trajectory, id, nam ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/name" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			const char *nam = (arg++)->AsString();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectName( Cursor, id, nam ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/name" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			const char *nam = (arg++)->AsString();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectName( Curve, id, nam ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/name" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			const char *nam = (arg++)->AsString();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectName( Trigger, id, nam ) ;
		}
		////////////////////////////////////////////////////////////////////
		///////////////// OBJECT CID
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/cid" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int cid = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectCID( Trajectory, id, cid ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/cid" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int cid = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectCID( Cursor, id, cid ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/cid" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int cid = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectCID( Curve, id, cid ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/cid" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int cid = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectCID( Trigger, id, cid ) ;
		}
		////////////////////////////////////////////////////////////////////
		///////////////// OBJECT GID
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/gid" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int gid = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectGID( Trajectory, id, gid ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/gid" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int gid = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectGID( Cursor, id, gid ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/gid" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int gid = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectGID( Curve, id, gid ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/gid" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int gid = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectGID( Trigger, id, gid ) ;
		}
		////////////////////////////////////////////////////////////////////
		///////////////// OBJECT ACTIVITY
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/act" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int trajAct = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectActivity( Trajectory, id, trajAct ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/act" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int cursAct = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectActivity( Cursor, id, cursAct ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/act" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int curvAct = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectActivity( Curve, id, curvAct ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/act" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int trigAct = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectActivity( Trigger, id, trigAct ) ;
		}
		////////////////////////////////////////////////////////////////////
		/////////////////// OSC HOST
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/host" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			const char *host = (arg++)->AsString();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectHost( Curve, id, host ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/host" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			const char *host = (arg++)->AsString();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectHost( Trigger, id, host ) ;
		}
		////////////////////////////////////////////////////////////////////
		/////////////////// OSC PORT
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/port" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int port = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectPort( Curve, id, port ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/port" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int id = (arg++)->AsInt32();
			int port = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscObjectPort( Trigger, id, port ) ;
		}
		////////////////////////////////////////////////////////////////////
		/////////////////// RENDER OBJECTS /////////////////////////////////
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/traject/show" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int straj = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscShowHideObject( Trajectory, straj ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/cursor/show" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int scurs = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscShowHideObject( Cursor, scurs ) ;
		}				
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/curve/show" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int scurv = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscShowHideObject( Curve, scurv ) ;
		}		
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/trigger/show" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int strig = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscShowHideObject( Trigger, strig ) ;
		}
		////////////////////////////////////////////////////////////////////
		///////////////// SCORE CONTROL
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/movecursors" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int mc = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscMoveCursors( mc ) ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/broadcast" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int broad = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscBroadcastMode( broad ) ;
		}	
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/osc" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			int oscm = (arg++)->AsInt32();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscOSCMode( oscm ) ;
		}	
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/play" ) == 0 ) {
			pScore->playPause() ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/stop" ) == 0 ) {
			pScore->pause() ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/reset" ) == 0 ) {
			pScore->stopAndReset() ;
		}
		else if( strcmp( m.AddressPattern(), "/iannix/ctrl/tempo" ) == 0 ) {
			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			float tempo = (arg++)->AsFloat();
			if( arg != m.ArgumentsEnd() ) throw osc::ExcessArgumentException();
				
			pScore->oscSetTempo( tempo ) ;
		}
		////////////////////////////////////////////////////////////////////
		///////////////// GET SCORE CONFIG
		if( strcmp( m.AddressPattern(), "/iannix/ctrl/info" ) == 0 ) {
			
			pScore->oscSendInfos() ;
		}
		////////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////////
	}
		
	catch( osc::Exception& e ) {
		printf("OSC Message Received but not interpreted (ERROR While Parsing) !!!\n") ;
	}
}


