///////////////
// IanniX
///////////////

////////////////////////////
// NxView.cpp
////////////////////////////

// Class Header
#include "NxView.h"

///////////////////////////////
// Subclass NxTabs
//////////////////////////////
NxTabs::NxTabs( QWidget* parent, QTabBar* tb )
  : QTabWidget(0)
{
	setTabBar( tb );
}

NxTabs::~NxTabs()
{}

//////////////////////////////////
// Subclass NxTabBar
/////////////////////////////////
NxTabBar::NxTabBar( QWidget* parent )
  : QTabBar( 0 )
{
	mainWindow = parent;
}

NxTabBar::~NxTabBar()
{}


////////////////////////////
// NxTabBar slots
////////////////////////////
void NxTabBar::contextMenuEvent( QContextMenuEvent* )
{
	QMenu* tabContext = new QMenu;
    
  	// Adding Items
  	tabContext->addAction( "&New Tab", 						this,		SLOT( newScore()) );
  	tabContext->addAction( "&Close Current Tab", 			this, 		SLOT( closeTab()) );
  	tabContext->addAction( "&Close All Tabs but Active", 	this, 		SLOT( closeButActive()) );
	tabContext->addAction( "&Close All Tabs",				this, 		SLOT( closeAllTabs()) );
	
  	tabContext->exec( QCursor::pos() );
  	delete tabContext;
}

void NxTabBar::setViewer(NxView *inViewer)
{
	viewer = inViewer;
}
void NxTabBar::newScore()
{
	viewer->newScore() ;
}
void NxTabBar::closeTab()
{
	viewer->closeScore();
}
void NxTabBar::closeAllTabs()
{
	viewer->closeAll();
}
void NxTabBar::closeButActive()
{
	viewer->closeAllButActive();
}

/////////////////////////////
// Class NxView
/////////////////////////////
NxView::NxView( QWidget* )
  : QMainWindow()
{
  	// Main Widget configuration
  	languageChange();
  	resize( QSize(800, 600) );
	move( 140, 30 ) ;
	setMinimumSize(790, 480) ;
	
	//QFont f( "Lucida Grande", 12);
	//setFont( f );
	
	//////////////// INITIALISATIONS
	noScoreOpened	= true ;
	allClosedSaved	= true ;
	
	antialiasing	= true ;
	
	totalNbScore	= 0 ;
  	index 			= 0 ;
  	settingsOk 		= true ;
	external 		= NULL ;
	copied			= NULL ;
	decObjectX		= 0 ;	
	incObjectID		= 1 ;

  	initMenuBar() ;
  	initToolBar() ;
  	initStatusBar() ;	
	initCloseDialog() ;
  	
	// Main Layout
	NxTabBar* myTabBar = new NxTabBar;
	myTabs = new NxTabs( this, myTabBar );
	myTabBar->setViewer(this);				
	
	setCentralWidget(myTabs);
	
  	// FileChooser
  	fileChooser = new QFileDialog;
  	fileChooser->setFilter( "XML files (*.xml *.html *.htm)" );
	
  	connect( myTabs, SIGNAL( currentChanged( QWidget* ) ),	this, SLOT( giveKeyboard(QWidget*) ) );
	connect( tempoSlider, SIGNAL( sliderReleased() ),		this, SLOT( reGiveHand() ) );
	connect( tempoSlider, SIGNAL( sliderMoved( int ) ),		this, SLOT( changeTempo( int ) ) );
	connect( tempoSlider, SIGNAL( actionTriggered( int ) ), this, SLOT( changeTempoTrigg( int ) ) );
}

/////////////////////
// Destructor
////////////////////
NxView::~NxView()
{
	GLwidgets.clear();
  	delete myTabs;
  	delete fileChooser;
}

///////////////////////////// PLAYBACK BUTTONS
void NxView::playScore() 
{		
	if(!noScoreOpened) GLwidgets.at(focusTab)->playPause() ;
}
void NxView::pauseScore() 
{		
	if(!noScoreOpened) GLwidgets.at(focusTab)->pause() ;
}
void NxView::resetScore() 
{
	if(!noScoreOpened) GLwidgets.at(focusTab)->stopAndReset() ;
}
void NxView::sendConfigOSC()
{
	if(!noScoreOpened) GLwidgets.at(focusTab)->sendConfigOSC() ;
}
//////////////////////////////////////
// ContextMenu Handler
/////////////////////////////////////
void NxView::contextMenuEvent( QContextMenuEvent*)
{
	/*
  	contextMenu = new QMenu;
  	// Items
  	contextMenu->addAction( "&New document", 	this, SLOT( newScore() ));
  	contextMenu->exec( QCursor::pos() );
	*/
}

////////////////////////////
// MENU BAR INIT
///////////////////////////
void NxView::initMenuBar()
{
  	// Popup Menus initialization
  	QMenu *file 		= new QMenu("File");
  	QMenu *edit 		= new QMenu("Edit");
  	QMenu *help 		= new QMenu("Help");
  	//QMenu *options 		= new QMenu("Options");
  	QMenu *modules 		= new QMenu("Modules");
	QMenu *display		= new QMenu("Display");
	QMenu *time			= new QMenu("Time");
	QMenu *osc			= new QMenu("OSC");
	
  	//////// File
	file->addAction( "&Preferences", 	this, SLOT( showPreferences() )		, Qt::CTRL + Qt::Key_P);
	file->addSeparator();
  	file->addAction( "&Open Score",				this, SLOT( openFromFile() ) , Qt::CTRL + Qt::Key_O);
  	file->addAction( "&New Score",				this, SLOT( newScore() ) , Qt::CTRL + Qt::Key_N);
	file->addAction( "&Close Current Score",	this, SLOT( closeScore() ) , Qt::CTRL + Qt::Key_W);
	file->addAction( "&Close All But Active", 	this, SLOT( closeAllButActive() ) );
  	file->addAction( "&Close All Scores",		this, SLOT( closeAll() ) );
  	file->addSeparator();
  	file->addAction( "&Save", 				this, SLOT( save() )  , Qt::CTRL + Qt::Key_S );
  	file->addAction( "&Save As",			this, SLOT( saveAs() ) , Qt::CTRL + Qt::SHIFT + Qt::Key_S );
  	file->addAction( "&Save All Scores",	this, SLOT( saveAll() ) );
	file->addSeparator();
	file->addAction( "&Quit",			this, SLOT( quitDialog() )			, Qt::CTRL + Qt::Key_Q);
  
  	//////// Edit
  	edit->addAction( "&Copy", 			this, SLOT( copy() )  , Qt::CTRL + Qt::Key_C );
  	edit->addAction( "&Cut", 			this, SLOT( cut() )  , Qt::CTRL + Qt::Key_X );
  	edit->addAction( "&Paste", 			this, SLOT( basicPaste() )  , Qt::CTRL + Qt::Key_V );
	edit->addAction( "&Duplicate", 		this, SLOT( duplicate() )  , Qt::CTRL + Qt::Key_D );
  	edit->addSeparator();
  	edit->addAction( "&Change Font", 	this, SLOT( font() ) );
	
  	//////// Modules
  	modules->addAction( "&Object Editor", 			this, SLOT( showObjectEditor() ) ,  Qt::CTRL + Qt::Key_E);
  	modules->addAction( "&Control Center", 			this, SLOT( showControl() )  , Qt::CTRL + Qt::Key_Z);
  	//modules->addAction( "&Advanced Render Module", 	this, SLOT( showAdvanced() ) );

	/*// Externals
	externals->addAction( "&Select Default software...", this, SLOT( tmp() ) );
	externals->addSeparator();
	externals->addAction( "&Launch PureData (with OSC library)", 	this, SLOT( launchPd() ) );
	externals->addAction( "&Launch Max/MSP", 						this, SLOT( launchMaxMSP() ) );*/
	
	////////// OSC Settings
	visualModeChoose = new QAction(tr("&Process Graphics"), this);
	connect(visualModeChoose, SIGNAL(triggered()), this, SLOT(switchVisualMode()) ) ;
	visualModeChoose->setCheckable(true) ;
	oscOutModeChoose = new QAction(tr("&Process OSC Out"), this);
	connect(oscOutModeChoose, SIGNAL(triggered()), this, SLOT(switchOSCoutMode()) ) ;
	oscOutModeChoose->setCheckable(true) ;
	oscInModeChoose = new QAction(tr("&Process OSC In"), this);
	connect(oscInModeChoose, SIGNAL(triggered()), this, SLOT(switchOSCinMode()) ) ;
	oscInModeChoose->setCheckable(true) ;
	broadcastChoose = new QAction(tr("&Broadcast mode"), this);
	connect(broadcastChoose, SIGNAL(triggered()), this, SLOT(switchBroadcastMode()) ) ;
	broadcastChoose->setCheckable(true) ;
	oscSendIFChoose = new QAction(tr("&Send Initial and Final OSC Messages"), this);
	connect(oscSendIFChoose, SIGNAL(triggered()), this, SLOT(switchSendIFmessages()) ) ;
	oscSendIFChoose->setCheckable(true) ;
	////////// TIME Settings
	algoChoose = new QAction(tr("&Use Circles Algorithm"), this);
	connect(algoChoose, SIGNAL(triggered()), this, SLOT(switchCirclesAlgo()) ) ;
	algoChoose->setCheckable(true) ;
	moveCursorsChoose = new QAction(tr("&Move Cursors"), this);
	connect(moveCursorsChoose, SIGNAL(triggered()), this, SLOT(switchMoveCursors()) ) ;
	moveCursorsChoose->setCheckable(true) ;
	algoChoose->setChecked(true) ;
	////////// DISPLAY Settings
	gridShow = new QAction(tr("&Show Grid"), this);
	connect(gridShow, SIGNAL(triggered()), this, SLOT(showHideGrid()) ) ;
	gridShow->setCheckable(true) ;
	crossesShow = new QAction(tr("&Show Center Crosses"), this);
	connect(crossesShow, SIGNAL(triggered()), this, SLOT(showHideCrosses()) ) ;
	crossesShow->setCheckable(true) ;	
	workspaceShow = new QAction(tr("&Show Workspace"), this);
	connect(workspaceShow, SIGNAL(triggered()), this, SLOT(showHideScore()) ) ;
	workspaceShow->setCheckable(true) ;	
	infosShow = new QAction(tr("&Show On Screen Infos"), this);
	connect(infosShow, SIGNAL(triggered()), this, SLOT(showHideScreenInfos()) ) ;
	infosShow->setCheckable(true) ;	
	circlesShow = new QAction(tr("&Show Curves Circles"), this);
	connect(circlesShow, SIGNAL(triggered()), this, SLOT(showHideCurvesCircles()) ) ;
	circlesShow->setCheckable(true) ;	
	curvesRectShow = new QAction(tr("&Show Curves Rectangles"), this);
	connect(curvesRectShow, SIGNAL(triggered()), this, SLOT(showHideCurvesRect()) ) ;
	curvesRectShow->setCheckable(true) ;	
	cursArea = new QAction(tr("&Show Cursors Impact Zones"), this);
	connect(cursArea, SIGNAL(triggered()), this, SLOT(showHideCursorsArea()) ) ;
	cursArea->setCheckable(true) ;	
	autoManualStep = new QAction(tr("&Auto Grid Step"), this);
	connect(autoManualStep, SIGNAL(triggered()), this, SLOT(switchAutoGridStep()) ) ;
	autoManualStep->setCheckable(true) ;	
	snapGrid = new QAction(tr("&Snap To Grid"), this);
	connect(snapGrid, SIGNAL(triggered()), this, SLOT(switchSnapToGrid()) ) ;
	snapGrid->setCheckable(true) ;	
	view3d = new QAction(tr("&3D View"), this);
	connect(view3d, SIGNAL(triggered()), this, SLOT(change2D3DView()) ) ;
	view3d->setCheckable(true) ;
	antialias = new QAction(tr("&Antialiasing"), this);
	connect(antialias, SIGNAL(triggered()), this, SLOT(switchAntialiasing()) ) ;
	antialias->setCheckable(true) ;
	antialias->setChecked(true) ;
	
	/////// OSC ///////////////////////////////////////////////////////////////////////////////////
	osc->addAction( broadcastChoose ) ;
	osc->addSeparator();
	osc->addAction( visualModeChoose ) ;
	osc->addAction( oscOutModeChoose ) ;
	osc->addAction( oscInModeChoose ) ;
	osc->addSeparator();
	osc->addAction( oscSendIFChoose ) ;
	osc->addSeparator();
	osc->addAction( "&Set Triggers/Curves Host IP and Port ...",	this, SLOT( setObjectsIPandPort() ) ) ;
	/////// TIME ///////////////////////////////////////////////////////////////////////////////////
	time->addAction( algoChoose ) ;
	time->addSeparator();
	time->addAction( moveCursorsChoose ) ;
	time->addSeparator();
	time->addAction( "&Change Curves Bounds ...",				this, SLOT( setCurvesBounds() ) ) ; 
	time->addAction( "&Flatten Curves ...",						this, SLOT( setCurvesFlatten() ) ) ; 
	time->addAction( "&Set Curves Scan Period ...",				this, SLOT( setCurvesSpatialPeriod() ) ) ; 
	time->addAction( "&Set Cursors Playback if Finished ...",	this, SLOT( setCursorsPlayFinished() ) ) ;
	time->addSeparator();
	time->addAction( "&Set Playback ...",	this, SLOT( setPlaybackTime() ) );
	time->addAction( "&Hard Offsets Reset",	this, SLOT( hardOffsetsReset() ) );
	time->addSeparator();
	time->addAction( "&Reset Speed Factor",		this, SLOT( resetTempo() )  , Qt::CTRL + Qt::Key_T);
	/////// DISPLAY ///////////////////////////////////////////////////////////////////////////////////
	display->addAction( "&Reset View",	this, SLOT( resetView() )  , Qt::CTRL + Qt::Key_R);
	display->addSeparator() ;
	display->addAction( workspaceShow ) ;
	display->addAction( infosShow ) ;
	display->addAction( crossesShow ) ;
	display->addAction( circlesShow ) ;
	display->addAction( curvesRectShow ) ;
	display->addAction( cursArea ) ;
	display->addSeparator();
	display->addAction( gridShow );
	display->addAction( autoManualStep );
	display->addAction( "&Set Grid Step ...",	this, SLOT( setGridStep() ) );
	display->addAction( snapGrid );
	display->addSeparator();
	display->addAction( view3d );
	display->addSeparator();
	display->addAction( antialias );
	/////// HELP ///////////////////////////////////////////////////////////////////////////////////
  	help->addAction( "&Keyboard Shortcuts", 	this, SLOT( showShortcuts() ) );
	QAction *tuto = new QAction(tr("&Tutorial"), this);
	connect(tuto, SIGNAL(triggered()), this, SLOT(launchTutorial()) ) ;
	tuto->setEnabled( false ) ;
	help->addAction( tuto ) ;
	help->addAction( "&About", 	this, SLOT( about() ) );
	////////////////////////////////////////////////////////////////////////////////////////////////
	menuBar()->addMenu(file);
	menuBar()->addMenu(edit);
	//menuBar()->addMenu(options);
	menuBar()->addMenu(modules);
	menuBar()->addMenu(osc);
	menuBar()->addMenu(time);
	menuBar()->addMenu(display);
	menuBar()->addSeparator();
	menuBar()->addMenu(help);
}

///////////////////
// ToolBar
///////////////////
void NxView::initToolBar()
{	
	////////// ICONS
	cpuOnIcon	= QIcon(":/pixmaps/cpuOn.png") ;
	cpuOffIcon	= QIcon(":/pixmaps/cpuOff.png") ;
	playIcon	= QIcon(":/pixmaps/play.png") ;
	pauseIcon	= QIcon(":/pixmaps/pause.png") ;
	stopIcon	= QIcon(":/pixmaps/stop.png") ;
	resetIcon	= QIcon(":/pixmaps/reset.png") ;
	configIcon	= QIcon(":/pixmaps/config.png") ;

	QToolBar *tBar = new QToolBar ;
	tBar->setMovable(false) ;

	QGroupBox *cpuGroup = new QGroupBox ;
	cpuGroup->setFixedSize(100,55) ; 
	QGroupBox *playbuttsGroup = new QGroupBox ;
	playbuttsGroup->setFixedSize(130,55) ; 
	QGroupBox *timeGroup = new QGroupBox ;
	timeGroup->setFixedSize(145,55) ;
	QGroupBox *tempoGroup = new QGroupBox ;
	tempoGroup->setFixedSize(135,55) ; 
	QGroupBox *renderGroup = new QGroupBox ;
	renderGroup->setFixedSize(190,55) ; 
	QGroupBox *configGroup = new QGroupBox ;
	configGroup->setFixedSize(55,55) ; 

	//QHBoxLayout *lay			= new QHBoxLayout(tBar) ;
	QHBoxLayout *cpuLay			= new QHBoxLayout(cpuGroup) ;
	QHBoxLayout *playbuttsLay	= new QHBoxLayout(playbuttsGroup) ;
	QHBoxLayout *timeLay		= new QHBoxLayout(timeGroup) ;
	QHBoxLayout *tempoLay		= new QHBoxLayout(tempoGroup) ;
	QGridLayout *renderLay		= new QGridLayout(renderGroup) ;
	QHBoxLayout *configLay		= new QHBoxLayout(configGroup) ;
	
	tbCpuValue	= new QLabel("0 ms") ;
	tbCpu		= new QToolButton ;
	playPause	= new QToolButton ;
	QToolButton *stop	= new QToolButton;
	QToolButton *reset	= new QToolButton;
	QToolButton *config = new QToolButton;
	tbTime		= new QLabel ;
	tbTempoF	= new QLineEdit(tempoGroup) ;
	tbTempoF->setMinimumHeight(20) ;
	tbTempo		= new QLabel("Speed :", tempoGroup ) ;
	tbInfoTxt	= new QLabel ;
	
	QLabel *rTrajectsL	= new QLabel( "Trajects", renderGroup ) ;
	QLabel *rCursorsL	= new QLabel( "Cursors", renderGroup ) ;
	QLabel *rCurvesL	= new QLabel( "Curves", renderGroup ) ;
	QLabel *rTriggersL	= new QLabel( "Triggers", renderGroup ) ;
	rTrajectsL->setScaledContents( true ) ;
	rCursorsL->setScaledContents( true ) ;
	rCurvesL->setScaledContents( true ) ;
	rTriggersL->setScaledContents( true ) ;
	rTrajectsC	= new QCheckBox( renderGroup ) ;
	rCursorsC	= new QCheckBox( renderGroup ) ;
	rCurvesC	= new QCheckBox( renderGroup ) ;
	rTriggersC	= new QCheckBox( renderGroup ) ;
	rTrajectsC->setChecked(true) ;
	rCursorsC->setChecked(true) ;
	rCurvesC->setChecked(true) ;
	rTriggersC->setChecked(true) ;
	
	cpuWarning = false ;
	tbCpu->setIcon( cpuOffIcon ) ;
    playPause->setIcon( playIcon ) ;
	stop->setIcon( stopIcon ) ;
	reset->setIcon( resetIcon ) ;
	config->setIcon( configIcon ) ;
	
	cpuLay->addWidget( tbCpu ) ;
	cpuLay->addWidget( tbCpuValue ) ;
	cpuGroup->setLayout( cpuLay ) ;
	playbuttsLay->addWidget( reset ) ;
	playbuttsLay->addWidget( stop ) ;
	playbuttsLay->addWidget( playPause ) ;
	playbuttsGroup->setLayout( playbuttsLay ) ;
	timeLay->addWidget( tbTime ) ;
	timeGroup->setLayout( timeLay ) ;
	tempoLay->addWidget( tbTempo ) ;
	tempoLay->addWidget( tbTempoF ) ;
	tempoGroup->setLayout( tempoLay ) ;
	renderLay->addWidget(rTrajectsC,	0,1) ;
	renderLay->addWidget(rTrajectsL,	0,0) ;
	renderLay->addWidget(rCursorsC,		1,1) ;
	renderLay->addWidget(rCursorsL,		1,0) ;
	renderLay->addWidget(rCurvesC,		0,3) ;
	renderLay->addWidget(rCurvesL,		0,2) ;
	renderLay->addWidget(rTriggersC,	1,3) ;
	renderLay->addWidget(rTriggersL,	1,2) ;
	renderGroup->setLayout( renderLay ) ;
	configLay->addWidget( config ) ;
	configGroup->setLayout( configLay ) ;
	//tBar->addWidget( cpuGroup ) ;
	tBar->addWidget( playbuttsGroup ) ;	
	tBar->addWidget( timeGroup ) ;
	tBar->addWidget( tempoGroup ) ;
	tBar->addWidget( renderGroup ) ;
	tBar->addWidget( configGroup ) ;
	//tBar->addStretch(50) ;
	//tBar->addWidget( tbInfoTxt ) ;
	//tBar->addSpacing(5) ;
	
	//delete(tBar->layout()) ;
	//tBar->setLayout(lay) ;
	
	connect(rTrajectsC, SIGNAL( released() ),	this, SLOT( showHideTrajects() ) ) ;
	connect(rCursorsC,	SIGNAL( released() ),	this, SLOT( showHideCursors() ) ) ;
	connect(rCurvesC,	SIGNAL( released() ),	this, SLOT( showHideCurves() ) ) ;
	connect(rTriggersC, SIGNAL( released() ),	this, SLOT( showHideTriggers() ) ) ;
	
	connect(tbCpu,		SIGNAL( clicked() ),			this, SLOT( showCPUinfo() ) );
	connect(tbTempoF,	SIGNAL( editingFinished() ),	this, SLOT( setTbTempo() ) );
	connect(reset,		SIGNAL( clicked()  ),			this, SLOT( resetScore() ) );
	connect(stop,		SIGNAL( clicked()  ),			this, SLOT( pauseScore() ) );
	connect(playPause,	SIGNAL( clicked()  ),			this, SLOT( playScore() ) );
	
	connect(config,		SIGNAL( clicked()  ),			this, SLOT( sendConfigOSC() ) );
	
	addToolBar(tBar) ;
}

//////////////////////
// Status Bar
//////////////////////
void NxView::initStatusBar()
{
  	// Creating slider
  	tempoSlider = new QSlider;
  	tempoSlider->setOrientation( Qt::Horizontal );
	
	// Default Tempo : 1.0
	tempoSlider->setRange(-1000, 1000);
	tempoSlider->setValue( 100 );
  	//tempoSlider->setTracking( true );
  	tempoSlider->setTickInterval( 10 );
	tempoSlider->setTickPosition(QSlider::NoTicks);

	//tempoSlider->setTickPosition(QSlider::TicksBelow);
	tempoSlider->setPalette( QPalette( QColor(155, 55, 0) ) ) ;
  	// Adding it to status bar
  	statusBar()->addWidget( tempoSlider, 1 );
}

void NxView::initCloseDialog()
{
	closeDialog = new QDialog ;
	closeDialog->setFixedSize(430,120) ;
	
	QGroupBox* buttG		= new QGroupBox(closeDialog);
	QVBoxLayout* vL 	= new QVBoxLayout ;
	QHBoxLayout* buttL 	= new QHBoxLayout ;
	buttG->setFlat( true ) ;
	
	txtL			= new QLabel("Do you want to save changes before closing ?", closeDialog ) ;
	quitB			= new QPushButton("Don't Save",			buttG );
	saveAndQuitB	= new QPushButton("Save All and Quit",	buttG );
	cancelB			= new QPushButton("Cancel",				buttG );
	
	buttL->addWidget(cancelB) ;
	buttL->addSpacing(65) ;
	buttL->addWidget(quitB) ;
	buttL->addWidget(saveAndQuitB) ;
	buttG->setLayout(buttL) ;
	txtL->setAlignment( Qt::AlignHCenter );
	vL->addWidget(txtL) ;
	vL->addWidget(buttG) ;
	closeDialog->setLayout(vL) ;
	
	//cancelB->setShortcut(Qt::Key_Escape) ;
	//quitB->setShortcut(Qt::Key_Space) ;
	//saveAndQuitB->setShortcut(Qt::Key_Return) ;
	
	closeDialog->setWindowModality( Qt::ApplicationModal ) ;
	
	connect( cancelB,		SIGNAL( clicked() ), closeDialog,	SLOT( close() ) ) ;
	connect( quitB,			SIGNAL( clicked() ), qApp,			SLOT( quit() ) ) ;
	connect( saveAndQuitB,	SIGNAL( clicked() ), this,			SLOT( saveAndQuit() ) ) ;
	quitDialogMode = true ;
}
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
// OPEN / CLOSE / SAVE SCORES - SLOTS
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
void NxView::newScore()
{
	QString str;
	openSettings();
	
  	if ( settingsOk ) {
  		// Opening Pop Up Menu for Settings
     	if ( partName.length() > 1 ) 
			str = partName;
		else {
			str = QString("NewScore%1").arg(totalNbScore+1);
			totalNbScore +=1 ;
		}
		
		launchGLBox( str, true );				
      	
		// Finally, updating score parameters
      	NxEnv tmpEnv = GLwidgets.at(index-1)->getPartition()->getEnvironment();
      	tmpEnv.name = str;
      	tmpEnv.ID = partID;
      	tmpEnv.length = partLength;
      	tmpEnv.height = partHeight;
		GLwidgets[index-1]->getPartition()->setEnvData( tmpEnv );
		GLwidgets[index-1]->updateIndexes();
    }
}
void NxView::openFromFile()
{
	QList<QString> XMLfilesToOpen ;
	
	XMLfilesToOpen = fileChooser->getOpenFileNames(
					0,
					"Select one or more files to open",
					".",
					"XML Files (*.xml)");

	for(int i=0; i<XMLfilesToOpen.size(); i++) 
	{
		setXMLinput( XMLfilesToOpen.at(i) );	
		launchGLBox( XMLfilesToOpen.at(i), false );
	}
}
/////////////////////////////////////////////////
// NEW SCORE SETTINGS
/////////////////////////////////////////////////
void NxView::setXMLinput( QString inFile )
{	
	inXMLfile = inFile;
}
void NxView::launchGLBox( QString inFileName, bool isNew )
{		
	NxGLScore* tmpBox = new NxGLScore( NULL, this, "My1stGLBox", inFileName, partLength, partHeight, isNew );
	GLwidgets << tmpBox;
	
  	myTabs->addTab( GLwidgets.last(), "tempName" );
	
	noScoreOpened = false ;
	
  	//printf( "Index New Page ! %i \n", index );
  	myTabs->setCurrentIndex( index );
  	giveKeyboard( GLwidgets.last() );
	
	GLwidgets.last()->fitToPageView() ;
	
	//////////// GET name and dimensions ////////////
	NxEnv tmpEnv = GLwidgets.last()->getPartition()->getEnvironment() ;
	QString nStr = QString("[%1x%2] ").arg(tmpEnv.length).arg(tmpEnv.height) ;
	nStr.append( tmpEnv.name ) ;
	myTabs->setTabText(focusTab, nStr) ;
	
	index++;
}
void NxView::openSettings()
{
	if (index > 0)
		GLwidgets[index-1]->releaseKeyboard();
  	
	opener = new QDialog;
  	opener->setWindowTitle( "New Score Settings" );	
	opener->setWindowModality( Qt::ApplicationModal ) ;
	
  	QVBoxLayout *setL = new QVBoxLayout;
	
  	///////// Groupboxes with layouts
  	QGroupBox* groupBox1 	= new QGroupBox(opener);
  	QGroupBox* groupBox2 	= new QGroupBox(opener);
  	QHBoxLayout* lay1 		= new QHBoxLayout;
  	QHBoxLayout* lay2 		= new QHBoxLayout;
	QHBoxLayout* butts 		= new QHBoxLayout;
		
  	///////// Widgets
  	QLabel* nameL 		= new QLabel( "Name", groupBox1 );
  	QLabel* IDL 		= new QLabel( "ID", groupBox1 );
  	QLineEdit* nameF 	= new QLineEdit( groupBox1 );							
  	QLineEdit* IDF 		= new QLineEdit( groupBox1 );
  	QLabel* lengthL 	= new QLabel( "Length (in s)", groupBox2 );
  	QLabel* heightL 	= new QLabel( "Height (in s)", groupBox2 );
  	QLineEdit* lengthF 	= new QLineEdit( groupBox2 );							
  	QLineEdit* heightF 	= new QLineEdit( groupBox2 );
	
  	QPushButton* validBtn = new QPushButton( "Create Score", opener );
	QPushButton* cancelBtn = new QPushButton( "Cancel", opener );
	
  	///////// Adding widgets
  	lay1->addWidget( nameL );
  	lay1->addWidget( nameF );
  	lay1->addWidget( IDL );
  	lay1->addWidget( IDF );
  	lay2->addWidget( lengthL );
  	lay2->addWidget( lengthF );
  	lay2->addWidget( heightL );
  	lay2->addWidget( heightF );
	butts->addWidget( cancelBtn ) ;
	butts->addWidget( validBtn );
  	setL->addWidget( groupBox1 );
  	setL->addWidget( groupBox2 );
  	setL->addLayout( butts ) ;
	
	groupBox1->setLayout(lay1);
	groupBox2->setLayout(lay2);
	opener->setLayout(setL);
	
  	nameL->setMaximumSize(70, 30);
  	IDL->setMaximumSize(70, 30);
  	lengthL->setMaximumSize(100, 30);
  	heightL->setMaximumSize(100, 30);
	//validBtn->setMaximumSize( 150, 30 );
	//cancelBtn->setMaximumSize( 150, 30 );
	
	//cancelBtn->setShortcut(Qt::Key_Escape) ;
	
	connect( cancelBtn, SIGNAL( clicked() ),	opener, SLOT( reject() ) );
  	connect( validBtn, SIGNAL( clicked() ),		opener, SLOT( accept() ) );
  	
	if ( opener->exec() == QDialog::Accepted ) {
    	bool ok;
      	settingsOk = true;
      	partName = nameF->text();
		
      	partID = IDF->text().toInt( &ok, 10 );
      	partLength = lengthF->text().toInt( &ok, 10 );
      	partHeight = heightF->text().toInt( &ok, 10 );
		if(partLength == 0) partLength = 10 ;
		if(partHeight == 0) partHeight = 10 ;
    }
  	else {
		settingsOk = false;
	}
}
/////////////////////////////////////////////////
// QUIT DIALOGS
/////////////////////////////////////////////////
void NxView::quitDialog()
{
	if(!noScoreOpened) {
		txtL->setText("Do you want to save changes before exit ?") ;
		saveAndQuitB->setText("Save All and Quit") ; 
		if(!quitDialogMode) {
			////// DISCONNECT SLOTS
			disconnect( quitB,			SIGNAL( clicked() ), closeDialog,	SLOT( accept() ) ) ;
			disconnect( saveAndQuitB,	SIGNAL( clicked() ), this,			SLOT( saveAndAccept() ) ) ;
			////// CONNECT SLOTS
			connect( quitB,			SIGNAL( clicked() ), qApp,	SLOT( quit() ) ) ;
			connect( saveAndQuitB,	SIGNAL( clicked() ), this,	SLOT( saveAndQuit() ) ) ;
			quitDialogMode = true ;
		}
		closeDialog->show() ;
	}
	else qApp->quit() ;
}
void NxView::saveAndQuit()
{
	closeDialog->accept() ;
	saveAll() ;
	if(allClosedSaved) qApp->quit() ;
}
void NxView::save()
{
	if(!noScoreOpened) {
		bool tmp = GLwidgets.at(focusTab)->save();
	}
}
void NxView::saveAs()
{
	if(!noScoreOpened) {
		bool tmp = GLwidgets.at(focusTab)->saveAs();
	}
}
void NxView::saveAll()
{
	allClosedSaved = true ;
	if(!noScoreOpened) {
		NxGLScore *tmpGLScore ;
		bool tmpsav ;
		for( int i=index-1; i>=0;i--) {
			myTabs->setCurrentIndex(i) ;
			tmpGLScore = (NxGLScore*)GLwidgets.at(i) ;
			giveKeyboard( (QWidget*)tmpGLScore ) ;
			tmpsav = ((NxGLScore*)tmpGLScore)->save() ;
			if(!tmpsav) allClosedSaved = false ;
		}
	}
	else allClosedSaved = false ;
}
void NxView::closeScoreDialog()
{
	if(!noScoreOpened) {
		int currFoc = myTabs->currentIndex();
		QString str = myTabs->tabText(currFoc) ;
		txtL->setText( QString("%1\nDo you want to save changes before closing ?").arg(str) ) ;
		saveAndQuitB->setText("Save and Close") ; 
		if(quitDialogMode) {
			////// DISCONNECT SLOTS
			disconnect( quitB,			SIGNAL( clicked() ), qApp,	SLOT( quit() ) ) ;
			disconnect( saveAndQuitB,	SIGNAL( clicked() ), this,	SLOT( saveAndQuit() ) ) ;
			////// CONNECT SLOTS
			connect( quitB,			SIGNAL( clicked() ), closeDialog,	SLOT( accept() ) ) ;
			connect( saveAndQuitB,	SIGNAL( clicked() ), this,			SLOT( saveAndAccept() ) ) ;
			quitDialogMode = false ;
		}
		closeDialog->show() ;
	
		closeOk = saveOk = false ;
	
		if ( closeDialog->exec() == QDialog::Accepted ) {
			closeOk = true ;
		}
		else {
			closeOk = false ;
		}
	}
}
void NxView::saveAndAccept()
{
	saveOk = true ;
	closeDialog->accept() ;
}
void NxView::closeScore()
{	
	if(!noScoreOpened) {
		closeScoreDialog() ;
	
		if(closeOk) {
			int tFocus = myTabs->currentIndex();
			bool savedSuceeded ;
			if(saveOk) 
				savedSuceeded = GLwidgets.at(tFocus)->save() ;
			
			if( savedSuceeded || !saveOk ) {
				GLwidgets.at(tFocus)->updateUDPListener( false ) ; // Stop OSC Listening
				myTabs->removeTab( tFocus );
				GLwidgets.removeAt(tFocus);
				index--;
				if(index==0) noScoreOpened = true ;
			}
		}
	}
}
void NxView::closeAll()
{	
	if(!noScoreOpened) {
		NxGLScore *tmpGLScore ;
		for( int i=index-1; i>=0 ; i-- ) {
			myTabs->setCurrentIndex(i) ;
			tmpGLScore = (NxGLScore*)GLwidgets.at(i) ;
			giveKeyboard( (QWidget*)tmpGLScore ) ;
			closeScore() ;
		}
	}
}
void NxView::closeAllButActive()
{
	if(!noScoreOpened) {
		int tFocus = myTabs->currentIndex();
	
		NxGLScore *tmpGLScore ;
		for( int i=index-1; i>tFocus ; i-- ) {
			myTabs->setCurrentIndex(i) ;
			tmpGLScore = (NxGLScore*)GLwidgets.at(i) ;
			giveKeyboard( (QWidget*)tmpGLScore ) ;
			closeScore() ;
		}
		for( int i=tFocus-1; i>=0 ; i-- ) {
			myTabs->setCurrentIndex(i) ;
			tmpGLScore = (NxGLScore*)GLwidgets.at(i) ;
			giveKeyboard( (QWidget*)tmpGLScore ) ;
			closeScore() ;
		}
		giveKeyboard( GLwidgets.last() );
	}
}
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
// CUT / COPY / PASTE / DUPLICATE - TOOLS
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
void NxView::cut()
{
	if(!noScoreOpened) {
		if (GLwidgets.at(focusTab)->getSelected() != NULL) {
			pauseScore() ;
			copied = GLwidgets.at(focusTab)->getSelected() ;
			GLwidgets.at(focusTab)->destroyObject( (NxObject*)GLwidgets.at(focusTab)->getSelected() ) ;
		}
		else
		QMessageBox::about(this, "Warning", "Nothing to cut") ;	
	}
}

void NxView::copy()
{
	if(!noScoreOpened) {
		incObjectID = 1 ;

		if (GLwidgets.at(focusTab)->getSelected() != NULL) {
			copied = GLwidgets.at(focusTab)->getSelected() ;
			/////// If trajectory, we have to copy the cursors too ...
			if( ((NxObject*)copied)->getObjectType()==Trajectory ) {
				if(cursCopied.size()!=0) cursCopied.clear() ;
				QList<int> indcur = ((NxTraject*)copied)->getCursorIDs() ;
				for(int i=0; i<indcur.count(); i++) { // save the cursors
					cursCopied.append( (NxCursor*)GLwidgets.at(focusTab)->getPartition()->getCursor(indcur.at(i)) ) ;
				}
			}
		}
		else
		QMessageBox::about(this, "Warning", "Nothing to copy") ;
	}
}

void NxView::basicPaste()
{
	paste(true, false) ;
}

void NxView::paste(bool incr, bool dec) // INCR : to increment the custom ID // DEC : to shift the object horizontaly
{
	if (copied != NULL && !noScoreOpened) {
	
		NxScore *tmpScore = GLwidgets.at(focusTab)->getPartition() ;
	
		// Common
		int incID = copied->getCustomID() ;
		int inGroupID = copied->getGroupID() ;
		int layer = copied->getLayer() ;
		QString inName = copied->getName() ;
		Color inColor1 = copied->getColor() ;
		// OSC
		QString inHost ;
		int inPort ;
		// Specific
		int inID , inIntValue, drawt, inSpatialPeriod, trajectType ;
		Color inColor2 ;
		QVector<NxPoint> editedCurve ;
		float realMin, realMax, scale, angle ;
		NxPoint inPos ; 
		inPos.ID = 0 ;						// initialisation
		inPos.x = inPos.y = inPos.z = 0 ;	// initialisation
	
		if(incr) {
			incID += incObjectID ;
			inName.append( QString("_c%1").arg(incObjectID) ) ;
			incObjectID += 1 ;
		}	
		if(dec) {
			decObjectX += GLwidgets.at(focusTab)->gridStepInt / 100.0 ;
			inPos.x = copied->getPosition().x + decObjectX ;
			inPos.y = copied->getPosition().y ;
			inPos.z = copied->getPosition().z ;
		}
		else inPos = GLwidgets.at(focusTab)->getCursor() ; 
	
		switch ( ((NxObject*)copied)->getObjectType() ) {
			case Curve: /////////////////////// curve
		
				//pauseScore() ;
			
				inID = GLwidgets.at(focusTab)->getPartition()->getCptCurve() ;
				inColor2			= ((NxCurve*)copied)->getCircleColor() ;
				editedCurve			= ((NxCurve*)copied)->getCurve() ;
				realMin				= ((NxCurve*)copied)->getRealMin() ;
				realMax				= ((NxCurve*)copied)->getRealMax() ;
				scale				= ((NxCurve*)copied)->getScale() ;
				angle				= ((NxCurve*)copied)->getAngle() ;
				drawt				= ((NxCurve*)copied)->getDrawType() ;
				inSpatialPeriod		= ((NxCurve*)copied)->getSpatialPeriod() ;
				inHost				= ((NxCurve*)copied)->getHost() ;
				inPort				= ((NxCurve*)copied)->getPort() ;
				tmpScore->addCurve( inPos, inID, incID, inGroupID, layer, inName, inColor1, inColor2, editedCurve, editedCurve.count(), realMin, realMax, scale, angle, inSpatialPeriod, inHost, inPort );
				GLwidgets.at(focusTab)->updateIndexes() ;
				((NxCurve*)tmpScore->getCurve( inID ))->setDrawType( drawt ) ;
				GLwidgets.at(focusTab)->getControlCenter()->updateItems( Curve, true, inID ) ;
				((NxCurve*)tmpScore->getCurve( inID ))->setPosition(inPos) ; // useful ?
				((NxCurve*)tmpScore->getCurve( inID ))->evalBoundingBox( inPos , 0 ) ;
				tmpScore->maxOfIndexes() ;
				GLwidgets.at(focusTab)->getScheduler()->addNewCurve() ;
			break;
			case Trajectory: /////////////////////// trajectory
			
				inID = GLwidgets.at(focusTab)->getPartition()->getCptTraject() ;
				trajectType = ((NxTraject*)copied)->getType() ;
				switch (trajectType) {
					case 1 : ////// Circle
						tmpScore->addTraject( inPos, inID, incID, inGroupID, layer, inName, inColor1, 1 );
						tmpScore->getTraject( inID )->setCircleDef( ((NxTraject*)copied)->getCircle() );
					break;		
					case 2 : ////// Line
						tmpScore->addTraject( inPos, inID, incID, inGroupID, layer, inName, inColor1, 2 );
						tmpScore->getTraject( inID )->setOrientation( ((NxTraject*)copied)->getOrientation() );
						tmpScore->getTraject( inID )->setLineDef( ((NxTraject*)copied)->getInitialPoint(), ((NxTraject*)copied)->getFinalPoint() );
					break;
				}
				tmpScore->getTraject( inID )->evalBoundingBox( inPos , GLwidgets.at(focusTab)->BBsize ) ;
				GLwidgets.at(focusTab)->updateIndexes();
				GLwidgets.at(focusTab)->getControlCenter()->updateItems( Trajectory, true, inID );
				tmpScore->maxOfIndexes() ;
				
				if(cursCopied.count()!=0) pauseScore() ;
				
				////////// We now have to copy the correspondant cursors /////////
				for(int i=0; i<cursCopied.count(); i++) {
					NxCursor *toCop = cursCopied.at(i) ;
					// Get cursor infos
					int newCursID = tmpScore->getCptCursor() ;
					QString newNam = toCop->getName().append("_c") ;
					// Add cursor
					tmpScore->addCursor( toCop->getPosition(), newCursID, toCop->getCustomID(), (NxTraject*)tmpScore->getTraject(inID), toCop->getGroupID(), toCop->getLayer(), newNam, toCop->getColor(), toCop->getForward(), toCop->getMode(), 0, toCop->getInitOffset(), toCop->getFinalOffset() );
					tmpScore->getCursor(newCursID)->reEvalOffsets() ;
					tmpScore->getCursor(newCursID)->setSpeed( toCop->getSpeed() );
					tmpScore->getCursor(newCursID)->setWidth( toCop->getWidth() );
					tmpScore->getCursor(newCursID)->setPlayFinished( toCop->getPlayFinished() ) ;
					tmpScore->getCursor(newCursID)->setOffset(  toCop->getOffset() ) ;
					tmpScore->getCursor(newCursID)->evalBoundingBox( toCop->getPosition(), GLwidgets.at(focusTab)->BBsize ) ;
					GLwidgets.at(focusTab)->updateIndexes();
					GLwidgets.at(focusTab)->assignCursors();
					GLwidgets.at(focusTab)->getControlCenter()->updateItems( Cursor, true, newCursID );	
					tmpScore->maxOfIndexes() ;
				}	
				if(cursCopied.count()!=0) GLwidgets.at(focusTab)->getScheduler()->resetSchedule() ;
			break;
			case Cursor: /////////////////////// cursor
				// DO NOTHING
			break;
			case Trigger: /////////////////////// trigger
		
				//pauseScore() ;
				
				inID = tmpScore->getCptTrigger() ;
				inHost = ((NxTrigger*)copied)->getHost() ;
				inPort = ((NxTrigger*)copied)->getPort() ;
				inIntValue = ((NxTrigger*)copied)->getValue() ;
				GLwidgets.at(focusTab)->addNewTrigger( inPos, inID, incID, inGroupID, layer, inName, inColor1, inHost, inPort, inIntValue ) ;
				tmpScore->maxOfIndexes() ;
			break;
		}
	}
	else
	QMessageBox::about(this, "Warning", "Nothing to paste") ;
}

void NxView::duplicate()
{
	if (copied != NULL && !noScoreOpened) {
		lastCopied = copied ;
	}
	copy() ;
	if (copied != NULL && !noScoreOpened) {
		if( lastCopied != copied ) decObjectX = 0 ;
	}
	if (copied != NULL && !noScoreOpened) paste(false, true) ;
}

// Temp func (DEBUG)
void NxView::tmp() {}

////////////////////////////////
// Keyboard handler
///////////////////////////////
void NxView::reGiveHand()
{
	if(!noScoreOpened) {
		//printf("hand\n") ;
		focusTab = myTabs->currentIndex();
		GLwidgets.at(focusTab)->setFocus() ;
	}
}
void NxView::giveKeyboard( QWidget* inW )
{
	if(!noScoreOpened) {
		for(int i=0 ; i<index ; i++) { ////// Release all keyboard focus and manage timers
			GLwidgets.at(i)->releaseKeyboard() ;
			GLwidgets.at(i)->releaseMouse() ;
			GLwidgets.at(i)->stopGLTimer() ;
			GLwidgets.at(i)->updateUDPListener( false ) ; // Stop OSC Listening
		}
		
		inW->setMouseTracking( true );
		inW->setFocus() ; // (Grab Keyboard too)

		focusTab = myTabs->currentIndex();
		GLwidgets.at(focusTab)->startGLTimer() ;
		tbTempoF->setText( QString("%1").arg( (float)((NxGLScore*)myTabs->widget(focusTab))->getPartition()->getTempo() ) ) ;
		tempoSlider->setValue(  int( ((NxGLScore*)myTabs->widget(focusTab))->getPartition()->getTempo() * 100 )  );
		this->setWindowTitle( QString("IanniX Viewer - %1 %2").arg( GLwidgets.at(focusTab)->getZoomScale() ).arg('%') );
		GLwidgets.at(focusTab)->updateCheckedMenus() ;
		setToolBarPlayback() ;
		GLwidgets.at(focusTab)->updateAntialiasing() ;
		((NxGLScore*)inW)->updateUDPListener( ((NxGLScore*)myTabs->widget(focusTab))->getPartition()->getEnvironment().oscInMode ) ; // Re-init OSC Listener if active
	
		//printf( "Debug tab : %i\n", focusTab );
	}
}
void NxView::setToolBarInfosTxt( QString inTxt )
{
	tbInfoTxt->setText( inTxt ) ;
}
void NxView::setToolBarTime(NxGLScore* score, float inTime ) // refresh time value
{
	if( score == GLwidgets.at(focusTab) )
	{
		tbMin	= (int)floor( inTime/60.0 ) ;
		tbSec	= (int)floor( (inTime - tbMin*60.0) ) ;
		tbMs	= (int)floor( (inTime - tbMin*60.0 - tbSec)*1000.0 );
		time = QString(" Time : %1:%2:%3 ")
					.arg(tbMin, 3, 10, QChar( QLatin1Char('0') ) )
					.arg(tbSec, 2, 10, QChar( QLatin1Char('0') ) )
					.arg(tbMs, 3, 10, QChar( QLatin1Char('0') ) ) ;
					
		tbTime->setText( time ) ;
	}
}
void NxView::setToolBarPlayback() ///////// set play / pause icons
{
	usleep(10000) ;
	if( GLwidgets.at(focusTab)->isPlaying() )
		playPause->setIcon( pauseIcon ) ;			
	else
		playPause->setIcon( playIcon ) ;
}
void NxView::setTbTempo()
{
	int tempTempo = (int)floor( tbTempoF->text().toFloat()*100.0) ;
	changeTempo( tempTempo ) ;
}
void NxView::changeTempo( int inTempo )
{
	if(!noScoreOpened) {
		( (NxGLScore*)myTabs->widget(focusTab) )->setTempo( (float) ( inTempo / 100.0) );

		GLwidgets.at(focusTab)->setFocus() ;
		tbTempoF->setText( QString("%1").arg( inTempo/100.0 ) ) ;
		tempoSlider->setValue( inTempo );
	}
}
void NxView::changeTempoTrigg( int inTempo )
{
	if(!noScoreOpened) {
		if(inTempo==3) {
			float newTempo = ((NxGLScore*)myTabs->widget(focusTab))->getPartition()->getTempo() + 0.1 ;
			( (NxGLScore*)myTabs->widget(focusTab) )->setTempo( newTempo );
			tbTempoF->setText( QString("%1").arg( newTempo ) ) ;
		}
		if(inTempo==4) {
			float newTempo = ((NxGLScore*)myTabs->widget(focusTab))->getPartition()->getTempo() - 0.1 ;
			( (NxGLScore*)myTabs->widget(focusTab) )->setTempo( newTempo );
			tbTempoF->setText( QString("%1").arg( newTempo ) ) ;
		}
	
		GLwidgets.at(focusTab)->setFocus() ;
	}
}
void NxView::resetTempo()
{
	if(!noScoreOpened) {
		( (NxGLScore*)myTabs->widget(focusTab) )->setTempo( 1.0 );

		GLwidgets.at(focusTab)->setFocus() ;
		tbTempoF->setText( "1" ) ;
		tempoSlider->setValue( 100 );
	}
}
void NxView::hardOffsetsReset() 
{
	if(!noScoreOpened) ( (NxGLScore*)myTabs->widget(focusTab) )->hardOffsetsReset() ;
}

/////////////////////////////////////////////////////////////////////
///////////////////////// MENUS SLOTS
/////////////////////////////////////////////////////////////////////
//////////////////////////////////
// SCORE Modules Display
//////////////////////////////////
void NxView::showObjectEditor() {
	if(!noScoreOpened) GLwidgets[focusTab]->openEditor();
}
void NxView::showControl() {
	if(!noScoreOpened) GLwidgets[focusTab]->openControl();
}
void NxView::showAdvanced() {
	if(!noScoreOpened) GLwidgets[focusTab]->open3DCenter();
}
//////////////////////////////////
// Externals Handle
//////////////////////////////////
void NxView::launchPd()
{
	/*external = new QProcess;
	
	external->start("sh", QStringList() << "OSCControl/launchPd");
	if ( !external->start() )
	{
		QMessageBox* errorBox = new QMessageBox(this);
  		errorBox->setIcon( QMessageBox::Critical );
		errorBox->setText( "Unable to launch PureData !! \nPlease install it first, or check your system paths..." );
		errorBox->show();
	}*/
}
void NxView::launchMaxMSP()
{
}
void NxView::launchExternal()
{
}
void NxView::showCPUinfo()
{
	QMessageBox::information( this, "CPU Infos" , 
		"The time value displayed (in ms) correspond\nto the scheduler step\n\n"
		"The CPU warning light is turned on when :\n\n"
		"- Computer is too slow (scheduler step has to be\nincreased)\n"
		"- At least one cursor is going too fast (loss of\ninformations / curve scan period)"
		) ;
}
void NxView::showShortcuts()
{
	QDialog* aboutBox = new QDialog(this);
	aboutBox->setWindowTitle("Shortcuts") ;
	//aboutBox->setFixedSize(490, 570) ;
	
	QPushButton* ok = new QPushButton("OK",aboutBox) ;
	QLabel* txt = new QLabel(aboutBox) ;
	
	QVBoxLayout* lay = new QVBoxLayout(aboutBox) ;
	lay->addWidget(txt) ;
	lay->addWidget(ok) ;
	
	txt->setText(
			"<b>Selection and Editing</b> <blockquote><br>"
			"<b>Shift</b>	-	Objects Selection<br>"
			"<b>Backspace</b>	-	Delete an Object<br>"
			"<b>I</b>	-	Lock / Unlock X Direction<br>"
			"<b>J</b>	-	Lock / Unlock Y Direction<br>"
			"<b>E</b>		-	Curves or Linear Trajects Edition Mode<br>"
			"<b>A</b>		-	Add a Point (curve edition)<br>"
			"<b>K</b>		-	Remove a Point (Curve Edition)<br><hr></blockquote>"
			"<b>Display</b> <blockquote><br>"
			"<b>-/+</b>			Zoom - / +<br>"
			"<b>Z</b>		-	Custom Zoom Rectangle<br>"
			"<b>X</b>		-	Line Rule Mode<br>"
			"<b>V</b>		-	2D / 3D View<br>"
			"<b>'Less'</b>		-	3D View Mouse Rotation<br>"
			"<b>1/2</b>		-	3D View Y-Rotation<br>"
			"<b>3/4</b>		-	3D View X-Rotation<br>"
			"<b>F</b>		-	Show / Hide Cursors Areas<br>"
			"<b>C</b>		-	Show / Hide Curves Circles<br>"
			"<b>M</b>		-	Show / Hide On-Screen Informations<br>"
			"<b>P</b>		-	Show / Hide Score<br>"
			"<b>G</b>		-	Show / Hide Grid<br>"
			"<b>S</b>		-	Snap to Grid Mode<br><hr></blockquote>"
			"<b>Playback</b> <blockquote><br>"
			"<b>Space</b>	-	Play / Pause<br>"
			"<b>W</b>		-	Reset Cursors Positions and Scheduler<br></blockquote>"
		) ;
	
	aboutBox->setLayout(lay) ;
	aboutBox->setWindowModality( Qt::ApplicationModal ) ;
	connect( ok, SIGNAL( clicked() ), aboutBox, SLOT( close() ) );
	aboutBox->show();
}
void NxView::about() 
{	
	QPixmap logoKitch( ":/pixmaps/logoKitch.png" );
	QPixmap logoCult( ":/pixmaps/logoCult.png" );
  	QDialog* aboutBox = new QDialog(this);
	aboutBox->setWindowTitle("About IanniX") ;
	aboutBox->setFixedSize(520, 630) ;
	
	QLabel* iconKitch = new QLabel(aboutBox) ;
	iconKitch->setPixmap(logoKitch) ;
	QLabel* iconCult = new QLabel(aboutBox) ;
	iconCult->setPixmap(logoCult) ;
	QLabel* icontxt = new QLabel(aboutBox) ;
	icontxt->setAlignment( Qt::AlignCenter );
	QLabel* txt = new QLabel(aboutBox) ;
	txt->setAlignment( Qt::AlignHCenter );
	QPushButton* ok = new QPushButton("OK",aboutBox) ;
	icontxt->setText(
		    "<b>IanniX v0.640 Beta</b><br>"
		    "<b>2007 (c) la kitchen</b><br>"
		) ;		
	txt->setText(
			"<b>Project Manager</b><br>"
			"Thierry Coduys<br><br>"
			"<b>Contributors</b><br>"
			"Gerard Pape (CCMIX)<br><br>"
			"<b>Developpers</b><br>"
			"Adrien Lefevre - v0.54<br>"
			"Guillaume Ferry - v0.60<br>"
			"Cyrill Duneau - v0.6xx<br>"
			"Pierre Jullian de la Fuente - v0.6xx<br><br><br>"
			"<b>Thanks to</b><br>"
			"Jean Bresson, Paul Crabbe, Cyrille Henry, Thomas Thiery, Romain Vuillet,<br>"
			"and all users for their observations and suggestions.<br><br><br>"
			"see<br>" 
			"<b>http://iannix.la-kitchen.fr/iannix.html</b><br>"
			"for more informations<br><br>"
		) ;
	//txt->setOpenExternalLinks( true ) ;
	
	QVBoxLayout* lay = new QVBoxLayout(aboutBox) ;
	QHBoxLayout* hlay = new QHBoxLayout(aboutBox) ;
	hlay->addWidget(iconKitch) ;
	hlay->addWidget(icontxt) ;
	hlay->addWidget(iconCult) ;

	lay->addLayout(hlay) ;
	lay->addWidget(txt) ;
	lay->addWidget(ok) ;
	
	aboutBox->setLayout(lay) ;
	aboutBox->setWindowModality( Qt::ApplicationModal ) ;
	
	//connect( txt, SIGNAL( linkActivated ( const QString & link) ), this, SLOT( launchWebLink( const QString & link ) ) );
	connect( ok, SIGNAL( clicked() ), aboutBox, SLOT( close() ) );
	aboutBox->show();
}
void NxView::launchWebLink( const QString & link )
{
	//bool tmp = QDesktopServices::openUrl( QUrl( link ) ) ;
}
void NxView::launchTutorial()
{

}
////////////////////////
// Font Chooser
////////////////////////
void NxView::font()
{
	bool ok;
  	QApplication::setFont( QFontDialog::getFont( &ok ) );	
}

void NxView::languageChange()
{
}

/////////////////////////////////////
// Preferences
////////////////////////////////////
void NxView::showPreferences()
{
	if(!noScoreOpened) {
		GLwidgets[focusTab]->releaseKeyboard();
		// Initializations
		prefMenu	= new QDialog ;
		prefMenu->setWindowTitle( "Score Settings" );
	
		QTabWidget* prefTabs 	= new QTabWidget;
	
		backColorPicked = false ;
		scoreColorPicked = false ;
		gridColorPicked = false ;
		pickedBackColor.ID = 0 ;
		pickedScoreColor.ID = 0 ;
		pickedGridColor.ID = 0 ;
	
		QVBoxLayout* vLay		= new QVBoxLayout; // Vertical Layout
		vLay->setAlignment( Qt::AlignHCenter );
			
		QHBoxLayout* hbutts			= new QHBoxLayout;
		QWidget* mainSettings		= new QWidget ; 
		QWidget* speedSettings		= new QWidget ;
		QWidget* networkSettings	= new QWidget ;
		QWidget* defaultSettings	= new QWidget ;
		QGroupBox* buttons			= new QGroupBox(prefMenu) ;
		
		QVBoxLayout* mainLay = new QVBoxLayout;		// Main + Colors + Grid Opacity
			QGridLayout* gridOpLay	= new QGridLayout;	// grid opacity
			QHBoxLayout* colorLay	= new QHBoxLayout;	// colors
			QGridLayout* mainLay1	= new QGridLayout;	// main
		QGridLayout* speedLay	= new QGridLayout;	// Speed
		QVBoxLayout* oscLay		= new QVBoxLayout;	// OSC
		QGridLayout* defaultLay = new QGridLayout;	// Default Settings
	
		QGroupBox*	oscGp	= new QGroupBox(networkSettings) ;
		oscGp->setFlat( true ) ;
		QGroupBox*	oscC	= new QGroupBox("OSC Curves Collisions Values", networkSettings) ;
		QGroupBox*	oscT	= new QGroupBox("OSC Triggers Collisions Values", networkSettings) ;
		QGridLayout* oscGpLay	= new QGridLayout(oscGp) ;
		QGridLayout* oscScLay	= new QGridLayout(oscC) ;
		QGridLayout* oscStLay	= new QGridLayout(oscT) ;
		
		//////////////////// Buttons
		QPushButton *apply 	= new QPushButton("Apply") ;
		QPushButton *cancel = new QPushButton("Cancel") ;
		hbutts->addWidget(cancel);
		hbutts->addWidget(apply);
		buttons->setLayout(hbutts) ;
		buttons->setFlat( true ) ;
		//////////////////// Main Settings
		////////// Grid Opacity Slider ////////////////////
		QLabel *gridOpL			= new QLabel("Grid Opacity", mainSettings );
		QSlider *gridOpSlider	= new QSlider ;
		gridOpSlider->setOrientation( Qt::Horizontal );
		gridOpSlider->setRange(2, 40);
		gridOpSlider->setTickPosition(QSlider::NoTicks);
		////////// Cursor ZONES Opacity Slider ////////////////////
		QLabel *zonesOpL		= new QLabel("Cursors Zones Opacity", mainSettings );
		QSlider *zonesOpSlider	= new QSlider ;
		zonesOpSlider->setOrientation( Qt::Horizontal );
		zonesOpSlider->setRange(2, 35);
		zonesOpSlider->setTickPosition(QSlider::NoTicks);
		// Colors
		chooseBackColor		= new QPushButton("Background Color", mainSettings );
		chooseScoreColor	= new QPushButton("Score Color", mainSettings ) ;
		chooseGridColor		= new QPushButton("Grid Color", mainSettings ) ;
		nameL 	= new QLabel("Score Name", mainSettings );
		nameF 	= new QLineEdit( mainSettings );
		IDL 	= new QLabel("ID", mainSettings );
		IDF 	= new QLineEdit( mainSettings );
		lengthL = new QLabel("Current Length (in s)", mainSettings );
		lengthF = new QLineEdit( mainSettings );
		heightL = new QLabel("Current Height (in s)", mainSettings );
		heightF = new QLineEdit( mainSettings );
		colorLay->addWidget(chooseBackColor) ;
		colorLay->addWidget(chooseScoreColor) ;
		colorLay->addWidget(chooseGridColor) ;
		mainLay1->addWidget( nameL,		0, 0 );
		mainLay1->addWidget( IDL, 		1, 0 );
		mainLay1->addWidget( lengthL, 	2, 0 );
		mainLay1->addWidget( heightL, 	3, 0 );
		mainLay1->addWidget( nameF,		0, 1 );
		mainLay1->addWidget( IDF, 		1, 1 );
		mainLay1->addWidget( lengthF, 	2, 1 );
		mainLay1->addWidget( heightF, 	3, 1 );
		gridOpLay->addWidget(gridOpL,		0, 0 );
		gridOpLay->addWidget(gridOpSlider,	0, 1 );
		gridOpLay->addWidget(zonesOpL,		1, 0 );
		gridOpLay->addWidget(zonesOpSlider, 1, 1 );
		mainLay->addLayout(gridOpLay) ;
		mainLay->addLayout(colorLay) ;
		mainLay->addLayout(mainLay1) ;
		mainSettings->setLayout(mainLay) ;
		
		//////////////////// OSC Settings	
		QLabel *configIPL	= new QLabel("OSC OUT Config Message IP", networkSettings );	
		QLabel *inportL		= new QLabel("OSC IN Port", networkSettings );
		QLabel *cOSCFormatL	= new QLabel("OSC Curves Collisions Format", networkSettings );
		QLabel *tOSCFormatL	= new QLabel("OSC Triggers Collisions Format", networkSettings );
		configIPF		= new QLineEdit( networkSettings );
		inPortF			= new QLineEdit( networkSettings );
		mformat1F		= new QLineEdit( networkSettings );
		mformat2F		= new QLineEdit( networkSettings );
		/// Curves / Triggers options
		oscScCRID	= new QCheckBox("Cursor ID", networkSettings ) ;
		oscScCID	= new QCheckBox("Cursor Custom ID", networkSettings ) ;
		oscScCGID	= new QCheckBox("Cursor Group ID", networkSettings ) ;
		oscScRID	= new QCheckBox("ID", networkSettings ) ;
		oscScID		= new QCheckBox("Custom ID", networkSettings ) ;
		oscScGID	= new QCheckBox("Group ID", networkSettings ) ;
		oscScA		= new QCheckBox("Angle", networkSettings ) ;
		oscScV		= new QCheckBox("Value", networkSettings ) ;
		oscScX		= new QCheckBox("X", networkSettings ) ;
		oscScY		= new QCheckBox("Y", networkSettings ) ;
		oscScZ		= new QCheckBox("Z", networkSettings ) ;
		oscStCRID	= new QCheckBox("Cursor ID", networkSettings ) ;
		oscStCID	= new QCheckBox("Cursor Custom ID", networkSettings ) ;
		oscStCGID	= new QCheckBox("Cursor Group ID", networkSettings ) ;
		oscStRID	= new QCheckBox("ID", networkSettings ) ;
		oscStID		= new QCheckBox("Custom ID", networkSettings ) ;
		oscStGID	= new QCheckBox("Group ID", networkSettings ) ;
		oscStV		= new QCheckBox("Int Value", networkSettings ) ;
		oscStX		= new QCheckBox("X", networkSettings ) ;
		oscStY		= new QCheckBox("Y", networkSettings ) ;
		oscStZ		= new QCheckBox("Z", networkSettings ) ;
		/// layout
		oscScLay->addWidget( oscScCRID, 0, 0 ) ;
		oscScLay->addWidget( oscScCGID, 1, 0 ) ;
		oscScLay->addWidget( oscScCID,	2, 0 ) ;
		oscScLay->addWidget( oscScRID,	0, 1 ) ;
		oscScLay->addWidget( oscScGID,	1, 1 ) ;
		oscScLay->addWidget( oscScID,	2, 1 ) ;
		oscScLay->addWidget( oscScA,	0, 2 ) ;
		oscScLay->addWidget( oscScV,	1, 2 ) ;
		oscScLay->addWidget( oscScX,	0, 3 ) ;
		oscScLay->addWidget( oscScY,	1, 3 ) ;
		oscScLay->addWidget( oscScZ,	2, 3 ) ;
		oscStLay->addWidget( oscStCRID, 0, 0 ) ;
		oscStLay->addWidget( oscStCGID, 1, 0 ) ;
		oscStLay->addWidget( oscStCID,	2, 0 ) ;
		oscStLay->addWidget( oscStRID,	0, 1 ) ;
		oscStLay->addWidget( oscStGID,	1, 1 ) ;
		oscStLay->addWidget( oscStID,	2, 1 ) ;
		oscStLay->addWidget( oscStV,	0, 2 ) ;
		oscStLay->addWidget( oscStX,	0, 3 ) ;
		oscStLay->addWidget( oscStY,	1, 3 ) ;
		oscStLay->addWidget( oscStZ,	2, 3 ) ;
		oscC->setLayout( oscScLay ) ;
		oscT->setLayout( oscStLay ) ;
		///
		oscGpLay->addWidget( configIPL,		0, 0 );
		oscGpLay->addWidget( inportL,		1, 0 );
		oscGpLay->addWidget( cOSCFormatL,	2, 0 );
		oscGpLay->addWidget( tOSCFormatL,	3, 0 );
		oscGpLay->addWidget( configIPF,		0, 1 );
		oscGpLay->addWidget( inPortF,		1, 1 );
		oscGpLay->addWidget( mformat1F,		2, 1 );
		oscGpLay->addWidget( mformat2F,		3, 1 );
		oscGp->setLayout( oscGpLay ) ;
		oscLay->addWidget(oscGp);
		oscLay->addWidget(oscC);
		oscLay->addWidget(oscT);
		networkSettings->setLayout(oscLay) ;
		
		//////////////////// TIME
		QLabel *tempoL 	= new QLabel("Speed", speedSettings );
		tempoF 	= new QLineEdit( speedSettings );
		QLabel *dStepL 	= new QLabel("Display FPS", speedSettings );
		dStepF 	= new QSpinBox( speedSettings );
		manualSchStep = new QCheckBox("Custom Scheduler Step (in ms)", speedSettings ) ;
		sStepF 	= new QSpinBox( speedSettings );
		////// Scheduler step deactivated
		manualSchStep->setEnabled(false) ;
		sStepF->setEnabled(false) ;
		dStepF->setMinimum(1) ;
		dStepF->setMaximum(60) ;
		sStepF->setMinimum(2) ;
		sStepF->setMaximum(100) ;
		speedLay->addWidget( tempoL, 0, 0 );
		speedLay->addWidget( tempoF, 0, 1 );
		speedLay->addWidget( dStepL, 1, 0 );
		speedLay->addWidget( dStepF, 1, 1 );
		speedLay->addWidget( manualSchStep, 2, 0 );
		speedLay->addWidget( sStepF, 2, 1 );
		speedSettings->setLayout(speedLay) ;
	
		//////////////////// DEFAULT VALUES
		hostL			= new QLabel("Host IP", defaultSettings );
		hostF			= new QLineEdit( defaultSettings );
		outportL		= new QLabel("UDP OSC Out Port", defaultSettings );
		outPortF		= new QLineEdit( defaultSettings );
		ivalueL			= new QLabel("OSC Triggers Int Value", defaultSettings );
		ivalueF			= new QLineEdit( defaultSettings );
		QLabel* mapp1L	= new QLabel("Curves Mapping Minimum", defaultSettings );
		QLabel* mapp2L	= new QLabel("Curves Mapping Maximum", defaultSettings );
		mapp1F			= new QLineEdit( defaultSettings );
		mapp2F			= new QLineEdit( defaultSettings );
		defaultSpatialPeriod	= new QSpinBox( defaultSettings ) ;
		QLabel *defaultSP		= new QLabel( "Curves Scan Period (in ms)" ) ;
		defaultSpatialPeriod->setMinimum( 5 ) ;
		defaultLay->addWidget( hostL,	0, 0 );
		defaultLay->addWidget( hostF,	0, 1 );
		defaultLay->addWidget( outportL,1, 0 );
		defaultLay->addWidget( outPortF,1, 1 );
		defaultLay->addWidget( ivalueL,	2, 0 );
		defaultLay->addWidget( ivalueF,	2, 1 );
		defaultLay->addWidget( mapp1L,	3, 0 );
		defaultLay->addWidget( mapp1F,	3, 1 );
		defaultLay->addWidget( mapp2L,	4, 0 );
		defaultLay->addWidget( mapp2F,	4, 1 );
		defaultLay->addWidget( defaultSP,			5, 0 );
		defaultLay->addWidget( defaultSpatialPeriod,5, 1 );
		defaultSettings->setLayout(defaultLay) ;
		
		prefTabs->addTab( mainSettings, "Score" ) ;
		prefTabs->addTab( networkSettings, "OSC" ) ;
		prefTabs->addTab( speedSettings, "Time" ) ;
		prefTabs->addTab( defaultSettings, "Default Values" ) ;
		
		vLay->addWidget(prefTabs) ;
		vLay->addWidget(buttons) ;
		prefMenu->setLayout( vLay ) ;
	
		QString str;
		NxEnv tempEnv = GLwidgets[focusTab]->getPartition()->getEnvironment() ;
		/////////////////////// MAIN
		gridOpSlider->setValue( GLwidgets[focusTab]->getGridOpacity()*100 );
		zonesOpSlider->setValue( GLwidgets[focusTab]->getCursorZonesOpacity()*100 );
		nameF->setText( tempEnv.name );
		heightF->setText( str.setNum( tempEnv.height ) );
		lengthF->setText( str.setNum( tempEnv.length ) );
		IDF->setText( str.setNum( tempEnv.ID ) );
		/////////////////////// SPEED
		tempoF->setText( str.setNum( tempEnv.globalTempo ) ) ;
		dStepF->setValue( (int)floor(1000/tempEnv.displayStep) ) ;
		sStepF->setValue( tempEnv.schedulerStep ) ;
		/////////////////////// OSC NETWORK
		configIPF->setText( tempEnv.OSCConfigMessageIP ) ;
		hostF->setText( tempEnv.UDPhost );
		inPortF->setText( str.setNum( tempEnv.UDPin ) );
		mformat1F->setText( tempEnv.OSCCurvesDefaultMFormat );
		mformat2F->setText( tempEnv.OSCTriggersDefaultMFormat );
		oscScCRID->setChecked( tempEnv.scCRID ) ;
		oscScCID->setChecked( tempEnv.scCID ) ;
		oscScCGID->setChecked( tempEnv.scCGID ) ;
		oscScRID->setChecked( tempEnv.scRID ) ;
		oscScID->setChecked( tempEnv.scID ) ;
		oscScGID->setChecked( tempEnv.scGID ) ;
		oscScA->setChecked( tempEnv.scA ) ;
		oscScV->setChecked( tempEnv.scV ) ;
		oscScX->setChecked( tempEnv.scX ) ;
		oscScY->setChecked( tempEnv.scY ) ;
		oscScZ->setChecked( tempEnv.scZ ) ;
		oscStCRID->setChecked( tempEnv.stCRID ) ;
		oscStCID->setChecked( tempEnv.stCID ) ;
		oscStCGID->setChecked( tempEnv.stCGID ) ;
		oscStRID->setChecked( tempEnv.stRID ) ;
		oscStID->setChecked( tempEnv.stID ) ;
		oscStGID->setChecked( tempEnv.stGID ) ;
		oscStV->setChecked( tempEnv.stV ) ;
		oscStX->setChecked( tempEnv.stX ) ;
		oscStY->setChecked( tempEnv.stY ) ;
		oscStZ->setChecked( tempEnv.stZ ) ;
		/////////////////////// DEFAULT VALUES
		mapp1F->setText( str.setNum( tempEnv.mappingDefaultMin ) ) ;
		mapp2F->setText( str.setNum( tempEnv.mappingDefaultMax ) ) ;
		defaultSpatialPeriod->setValue( tempEnv.defaultSpatialPeriod ) ;
		ivalueF->setText( str.setNum( tempEnv.OSCTriggersDefaultIntValue ) );
		outPortF->setText( str.setNum( tempEnv.UDPout ) );
	
		prefMenu->setFixedSize( 460, 450 );
		prefMenu->setWindowModality( Qt::ApplicationModal ) ;
		apply->setShortcut(Qt::Key_Return) ;
		manualSchStep->setChecked( !GLwidgets[focusTab]->getScheduler()->isAutoSchedulerStep() ) ;
		sStepF->setEnabled( !GLwidgets[focusTab]->getScheduler()->isAutoSchedulerStep() ) ;
	
		/// Deactive manual scheduler step
		//manualSchStep->setEnabled( false ) ;
		//sStepF->setEnabled( false ) ;
		
		connect( gridOpSlider, SIGNAL( sliderMoved(int) ), this, SLOT( setGridOpacity(int) ) ) ;
		connect( zonesOpSlider, SIGNAL( sliderMoved(int) ), this, SLOT( setCursorZonesOpacity(int) ) ) ;
		
		connect( manualSchStep, SIGNAL( toggled(bool) ), sStepF, SLOT( setEnabled(bool) ) ) ;
		connect( chooseScoreColor, 	SIGNAL( clicked() ), this, SLOT( pickScoreColor() ) ) ;
		connect( chooseBackColor, 	SIGNAL( clicked() ), this, SLOT( pickBackColor() ) ) ;
		connect( chooseGridColor, 	SIGNAL( clicked() ), this, SLOT( pickGridColor() ) ) ;
		connect( cancel, SIGNAL( clicked() ), prefMenu, SLOT( close() ) ) ;
		connect( apply,	SIGNAL( clicked() ), this, SLOT( setPreferences() ) ) ;
	
		prefMenu->show() ;
	}
}
void NxView::pickScoreColor() 
{
	pickedScoreColor.c = colorPicker->getColor();	
	scoreColorPicked = true ;
}
void NxView::pickBackColor() 
{	
	pickedBackColor.c = colorPicker->getColor();	
	backColorPicked = true ;
}
void NxView::pickGridColor() 
{	
	pickedGridColor.c = colorPicker->getColor();	
	gridColorPicked = true ;
}
void NxView::setGridOpacity( int inOp ) 
{	
	GLwidgets[focusTab]->setGridOpacity( inOp/100.0 ) ;
}
void NxView::setCursorZonesOpacity( int inOp ) 
{
	GLwidgets[focusTab]->setCursorZonesOpacity( inOp/100.0 ) ;
}
void NxView::setPreferences()
{
	bool resize = false ;
	NxEnv tmpEnv = GLwidgets[focusTab]->getPartition()->getEnvironment();
	/////////////////////// MAIN
	if(scoreColorPicked)	tmpEnv.scoreColor = pickedScoreColor ;
	if(backColorPicked)		tmpEnv.backColor = pickedBackColor ;
	if(gridColorPicked)		tmpEnv.gridColor = pickedGridColor ;
	tmpEnv.name = nameF->text();
	tmpEnv.ID = IDF->text().toInt();
	int nL = lengthF->text().toInt() ;
	int nH = heightF->text().toInt() ;
	if( tmpEnv.length != nL || tmpEnv.height != nH ) resize = true ;
	tmpEnv.length = nL ;
	tmpEnv.height = nH ;
	/////////////////////// SPEED
	tmpEnv.globalTempo = tempoF->text().toFloat() ; 
	tmpEnv.displayStep = (int)floor( 1000.0/(dStepF->value()) ) ;
	if( manualSchStep->isChecked() ) {
		tmpEnv.schedulerStep = sStepF->value() ;
		GLwidgets[focusTab]->getScheduler()->useAutoSchedulerStep( false ) ;
	}
	else GLwidgets[focusTab]->getScheduler()->useAutoSchedulerStep( true ) ;
	tmpEnv.defaultSpatialPeriod = defaultSpatialPeriod->value() ;
	/////////////////////// OSC NETWORK
	tmpEnv.UDPhost = hostF->text() ;
	tmpEnv.UDPin = inPortF->text().toInt() ;
	tmpEnv.UDPout = outPortF->text().toInt() ;
	tmpEnv.OSCCurvesDefaultMFormat = mformat1F->text() ;
	tmpEnv.OSCTriggersDefaultMFormat = mformat2F->text() ;
	tmpEnv.OSCTriggersDefaultIntValue = ivalueF->text().toInt() ;
	tmpEnv.OSCConfigMessageIP = configIPF->text() ;
	// osc sending options (what to send)
	tmpEnv.scCRID	= oscScCRID->isChecked() ;
	tmpEnv.scCID	= oscScCID->isChecked() ;
	tmpEnv.scCGID	= oscScCGID->isChecked() ;
	tmpEnv.scRID	= oscScRID->isChecked() ;
	tmpEnv.scID		= oscScID->isChecked() ;
	tmpEnv.scGID	= oscScGID->isChecked() ;
	tmpEnv.scA		= oscScA->isChecked() ;
	tmpEnv.scV		= oscScV->isChecked() ;
	tmpEnv.scX		= oscScX->isChecked() ;
	tmpEnv.scY		= oscScY->isChecked() ;
	tmpEnv.scZ		= oscScZ->isChecked() ;
	tmpEnv.stCRID	= oscStCRID->isChecked() ;
	tmpEnv.stCID	= oscStCID->isChecked() ;
	tmpEnv.stCGID	= oscStCGID->isChecked() ;
	tmpEnv.stRID	= oscStRID->isChecked() ;
	tmpEnv.stID		= oscStID->isChecked() ;
	tmpEnv.stGID	= oscStGID->isChecked() ;
	tmpEnv.stV		= oscStV->isChecked() ;
	tmpEnv.stX		= oscStX->isChecked() ;
	tmpEnv.stY		= oscStY->isChecked() ;
	tmpEnv.stZ		= oscStZ->isChecked() ;
	/////////////////////// MAPPING
	tmpEnv.mappingDefaultMin = mapp1F->text().toFloat() ;
	tmpEnv.mappingDefaultMax = mapp2F->text().toFloat() ;
	
	GLwidgets[focusTab]->getPartition()->setEnvData( tmpEnv );
	GLwidgets[focusTab]->stopGLTimer() ;
	GLwidgets[focusTab]->startGLTimer() ; ////// refresh the display step
	GLwidgets[focusTab]->updateIndexes();
	
	int tFocus = myTabs->currentIndex();
	QString nStr = QString("[%1x%2] ").arg(tmpEnv.length).arg(tmpEnv.height) ;
	nStr.append( tmpEnv.name ) ;
	myTabs->setTabText(tFocus, nStr) ;
	tempoSlider->setValue( int( tmpEnv.globalTempo*100 ) ) ;
	
	GLwidgets[focusTab]->updateScoreColors() ;
	////////////// OSC UDP Listener
	GLwidgets[focusTab]->updateUDPListener( false ) ;
	if(tmpEnv.oscInMode) GLwidgets[focusTab]->updateUDPListener( true ) ;
	////////////// OSC What to Send
	GLwidgets[focusTab]->getScheduler()->updateSendingOptions() ;
	
	if(resize) GLwidgets[focusTab]->hardResetView() ;
	
	prefMenu->accept() ;
}
///////////////////////////////////////////////
///////////////////////////////////////////////
/////////////// CPU Light Warning /////////////
void NxView::setCPUWarning( bool act )
{
	if(!cpuWarning && act) {
		tbCpu->setIcon( cpuOnIcon ) ;
		cpuWarning = true ;
		}
	if(cpuWarning && !act) {
		tbCpu->setIcon( cpuOffIcon ) ;
		cpuWarning = false ;
		}
}
void NxView::setCPUTimeStep( int val )
{
	tbCpuValue->setText( QString("%1 ms").arg( val ) ) ;
}
///////////////////////////////////////////////
//////// Settings Shortcuts ///////////////////
///////////////////////////////////////////////
bool NxView::getAntialiasing()
{
	return antialiasing ;
}
void NxView::updateCheckedMenus( bool algo, bool moveC, bool broadcM, bool visuM, bool oscifM, bool oscOutM, bool oscInM, bool snap, bool gridS, bool autostep, bool trajS, bool cursS, bool curvS, bool trigS, bool crossesS, bool circlesS, bool rectS, bool cursAS, bool infosS, bool workspaceS, bool viewtype )
{	
	////////////////////////////////////////// RENDERING
	rTrajectsC->setChecked( trajS ) ;
	rCursorsC->setChecked( cursS ) ;
	rCurvesC->setChecked( curvS ) ;
	rTriggersC->setChecked( trigS ) ;
	////////////////////////////////////////// TIME / OSC
	algoChoose->setChecked( algo ) ;
	oscSendIFChoose->setChecked( oscifM ) ;
	moveCursorsChoose->setChecked( moveC ) ;
	autoManualStep->setChecked( autostep ) ;
	broadcastChoose->setChecked( broadcM ) ;
	////////////////////////////////////////// MODES
	visualModeChoose->setChecked( visuM ) ;
	oscOutModeChoose->setChecked( oscOutM ) ;
	oscInModeChoose->setChecked( oscInM ) ;
	////////////////////////////////////////// DISPLAY
	snapGrid->setChecked( snap ) ;
	gridShow->setChecked( gridS ) ;
	crossesShow->setChecked( crossesS ) ;
	circlesShow->setChecked( circlesS ) ;
	curvesRectShow->setChecked( rectS ) ;
	cursArea->setChecked( cursAS ) ;
	infosShow->setChecked( infosS ) ;
	workspaceShow->setChecked( workspaceS ) ;
	view3d->setChecked( viewtype ) ;
}
void NxView::switchCirclesAlgo() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchCirclesAlgo() ;
}
void NxView::switchSendIFmessages() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchSendIFmessages() ;
}
void NxView::switchMoveCursors() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchMoveCursors() ;
}
void NxView::switchBroadcastMode() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchBroadcastMode() ;
}
void NxView::switchVisualMode() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchVisualMode() ;
}
void NxView::switchOSCoutMode() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchOSCoutMode() ;
}
void NxView::switchOSCinMode() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchOSCinMode() ;
}
void NxView::setPlaybackTime() {
	if(!noScoreOpened) GLwidgets[focusTab]->changePlaybackTime() ;
}
void NxView::setObjectsIPandPort() {
	if(!noScoreOpened) GLwidgets[focusTab]->changeGlobalIPandPort() ;
}
void NxView::setCurvesSpatialPeriod() {
	if(!noScoreOpened) GLwidgets[focusTab]->changeGlobalSpatialPeriod() ;
}
void NxView::setCurvesBounds() {
	if(!noScoreOpened) GLwidgets[focusTab]->changeGlobalCurvesBounds() ;
}
void NxView::setCurvesFlatten() {
	if(!noScoreOpened) GLwidgets[focusTab]->changeGlobalCurvesFlatten() ;
}
void NxView::setCursorsPlayFinished() {
	if(!noScoreOpened) GLwidgets[focusTab]->changeGlobalCursorFinished() ;
}
void NxView::change2D3DView() {
	if(!noScoreOpened) GLwidgets[focusTab]->change2D3DView() ;
}
void NxView::switchAntialiasing() {
	antialiasing = !antialiasing ;
	if(!noScoreOpened) GLwidgets[focusTab]->updateAntialiasing() ;
}
////////////////////////////////////////// Render Objects
void NxView::showHideTrajects() {
	if(!noScoreOpened) {
		GLwidgets[focusTab]->showHideTrajects() ;
		GLwidgets[focusTab]->setFocus() ;
	}
}
void NxView::showHideCursors() {
	if(!noScoreOpened) {
		GLwidgets[focusTab]->showHideCursors() ;
		GLwidgets[focusTab]->setFocus() ;
	}
}
void NxView::showHideCurves() {
	if(!noScoreOpened) {
		GLwidgets[focusTab]->showHideCurves() ;
		GLwidgets[focusTab]->setFocus() ;
	}
}
void NxView::showHideTriggers() {
	if(!noScoreOpened) {
		GLwidgets[focusTab]->showHideTriggers() ;
		GLwidgets[focusTab]->setFocus() ;
	}
}
///////////////////////////////////////////
void NxView::showHideCrosses() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideCrosses() ;
}
void NxView::showHideGrid() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideGrid() ;
}
void NxView::showHideScore() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideScore() ;
}
void NxView::showHideCurvesCircles() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideCurvesCircles() ;
}
void NxView::showHideCurvesRect() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideCurvesRect() ;
}
void NxView::showHideCursorsArea() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideCursorsArea() ;
}
void NxView::showHideScreenInfos() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideScreenInfos() ;
}
void NxView::switchSnapToGrid() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchSnapToGrid() ;
}
void NxView::switchAutoGridStep() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchAutoGridStep() ;
}
void NxView::setGridStep() {
	if(!noScoreOpened) GLwidgets[focusTab]->changeGridStep() ;
}
void NxView::resetView() {
	if(!noScoreOpened) GLwidgets[focusTab]->fitToPageView() ;
}




